#!python

"""
The overloading of the * operator in PyViennaCL is not directly compatible with
the usage in NumPy. In PyViennaCL, given the emphasis on linear algebra, we have
attempted to make the usage of * as natural as possible.

The semantics are as follows:
* Scalar * scalar -> scalar;
* scalar * vector -> scaled vector;
* scalar * matrix -> scaled matrix;
* vector * vector -> undefined;
* vector * matrix -> undefined;
* matrix * vector -> matrix-vector product;
* matrix * matrix -> matrix-matrix product.

Of course, there exist other products in some cases, which is why the * operator
is sometimes undefined. For instance, in the case of `vector * vector`, we
could have either a dot product, an outer product, a cross product, or an
elementwise product. The cross product is not currently implemented in
PyViennaCL, but this still leaves three cases from which to choose.

Here, we demonstrate the different notation for these products.
"""

import pyviennacl as p
import numpy as np

# Let's construct some random 1-D and 2-D arrays
v = np.random.rand(5)
w = np.random.rand(5)

f = np.random.rand(5, 5)
g = np.random.rand(5, 5)

# Now transfer them to the compute device
x, y = p.Vector(v), p.Vector(w)
a, b = p.Matrix(f), p.Matrix(g)

print("a is\n%s" % a)
print("b is\n%s" % b)

print("x is %s" % x)
print("y is %s" % y)

#
# Scaling
#

# Represent the scaling of x by 2.0
z = x * 2.0

# Compute and print the result
print("x * 2.0 = %s" % z)

# Represent the scaling of a by 2.0
c = a * 2.0

# Compute and print the result
print("a * 2.0 =\n%s" % c)

#
# Vector products
#

# Represent the dot product of x and y
d = x.dot(y) # or p.dot(x, y)

# Compute the dot product and print it
print("Dot product of x and y is %s" % d)

# Represent the elementwise product of x and y
z = x.element_prod(y) # or x.element_mul(y)

# Compute and print the result
print("Elementwise product of x and y is %s" % z)

# Represent the outer product of x and y
c = x.outer(y)

# Compute and print the result
print("Outer product of x and y:\n%s" % c)


#
# Matrix and matrix-vector products
#

# Represent the elementwise product of a and b
c = a.element_prod(b) # or a.elementwise_mul(b)

# Compute and print the result
print("Elementwise product of a and b:\n%s" % c)

# Represent the matrix product of a and b
c = a * b

# Compute and print the result
print("Matrix product of a and b:\n%s" % c)

# Represent the matrix-vector product of a and x
c = a * x

# Compute and print the result
print("Matrix-vector product of a and x:\n%s" % c)






#!python

"""
In this example, we investigate the construction and basic usage of
PyViennaCL's dense matrix (Matrix) and Vector types, and discuss some
important issues about integration with Python and NumPy data types, and
PyViennaCL's computational architecture.

If you are familiar with NumPy, you might need about 5 minutes to grasp the
content here. If you are not so familiar, you might need 10 minutes.
"""

# Import PyViennaCL and NumPy
import pyviennacl as p
import numpy as np

# Create our datastructures on the host
x = [1.0, 2.0, 3.0, 4.0, 5.0] # We can create PyViennaCL Vectors from lists
a = np.array([[1.0, 2.0, 3.0],
              [0.0, 3.0, 4.0],
              [0.0, 0.0, 5.0]]) # We can create PyViennaCL Matrices from arrays

# Create corresponding ViennaCL datastructures on the compute device
y = p.Vector(x)
b = p.Matrix(a) # This is a dense matrix

# Copy the data back to the host and check that it's equal
z = y.value # z is now a 1-D numpy array with dtype float64
c = b.value # c is now a 2-D numpy array with dtype float64

if (z == x).all() and (c == a).all():
    print("Successfully transferred data to and from the compute device!")

# We can modify elements of the ViennaCL device structures, but since this 
# incurs a compute kernel initialisation and buffer transfer, it is very slow!
y[0] = 0.0
b[2, 2] = -1.0

x[0] = 0.0     # We should also modify our original data to keep track..
a[2, 2] = -1.0

# And we can do comparisons seamlessly between NumPy and PyViennaCL types!
if (a == b).all() and (x == y).all():
    print("Successfully modified data structures on host and device!")

# We also need to be sure that we are consistent with respect to the data-types
# we use. For instance, we should not mix integer and floating point types.
#
# By default, PyViennaCL objects inherit the dtype of the objects from which
# they are created, or (if that is ambiguous), float64.
print("a and b have dtypes of %s and %s" %
      (np.result_type(a), np.result_type(b)))

# PyViennaCL makes an effort to convert objects to the appropriate dtype where
# dtypes have been mixed, but this is often ambiguous and unpredictable, and
# so it is recommended that users make efforts to keep they dtypes consistent.
i = 1L # Create a long integer
print("i has dtype of %s" % (np.result_type(i)))
y[0] = i # Remember, this sort of elementwise assignation is *very slow*!
print("y has values %s and dtype %s" %
      (y, np.result_type(y)))

# And, of course, we can perform basic arithemetic operations with PyViennaCL,
# mixing native Python types with NumPy and PyViennaCL types:
z = (x + y + z) / 2.0
print("z is now of type %s, dtype %s, and with values %s" 
      % (type(z), np.result_type(z), z))
# Notice that z has `Div' type. This is because the z object represents the
# arithmetic expression `(x + y + z) / 2.0', and this is only computed when
# the result is needed, in order to maximise performance.

# And we can do less basic arithmetic!
print("The sine of the values of z is %s" %
      p.sin(z))
# PyViennaCL exposes many elementwise mathematical functions.
# See help(p.math) for more information. 



#!python

"""
PyViennaCL provides direct solvers for dense triangular linear systems.
The API is documented in ``help(pyviennacl.linalg.solve)``. In particular,
the solver to use is determined by the tag instance supplied to the ``solve``
function.

LU factorisation is planned for a later release.

Here, we demonstrate the use of the direct triangular solver for an
upper triangular system; other forms and solvers are supported by the choice
of a different solver tag class.
"""

import pyviennacl as p
import numpy as np
import random

# We want a square N x N system.
N = 5 

# Create a NumPy matrix with float32 precision to hold the data on the host.
# Firstly, we create an empty matrix, then fill the upper triangle with values.
A = np.zeros((N, N), dtype = np.float32)

for i in range(N):
    for j in range(N):
        if j >= i:
            A[i, j] = np.float32(random.randint(0,1000) / 100.0)

# Transfer the system matrix to the compute device
A = p.Matrix(A)

print("A is\n%s" % A)

# Create a right-hand-side vector on the host with random elements
# and transfer it to the compute device
b = p.Vector(np.random.rand(N).astype(np.float32))

print("b is %s" % b)

# Solve the system; note the choice of tag to denote an upper triangular system
x = p.solve(A, b, p.upper_tag())

# Copy the solution from the device to host and display it
print("Solution of Ax = b for x:\n%s" % x)




#!/usr/bin/env python

import math
import numpy as np
import os
import pyviennacl as p
import sys

from test_common import diff, test_vector_slice

#TODO: Change print statements to log statements

def run_test(*args, **kwargs):
    """
    A and B must be numpy array or matrix instances with one dimension
    """
    epsilon = args[0]
    A = args[1]
    B = args[2]
    C = args[3]
    vcl_A = args[4]
    vcl_B = args[5]
    vcl_C = args[6]

    dtype = np.result_type(kwargs['dtype']).type

    alpha = p.Scalar(dtype(3.1415)) ## TODO SHOULD BE GPU SCALAR
    beta = p.HostScalar(dtype(2.718))

    ###
    ### TODO MISSING:
    ### + cpu / gpu combos
    ### + elementwise power function?
    ###

    # Test initialisers
    # + GPU scalar TODO
    #X = p.Vector(A.shape, alpha)
    #if not (X == (np.ones(A.shape, dtype = dtype) * alpha.value)).all():
    #    raise RuntimeError("Failed: GPU scalar vector init")
    #print("Test: initialisation of vector with GPU scalar passed")

    # + CPU scalar TODO
    Y = p.Vector(A.shape[0], beta.value) # TODO
    if not (Y == (np.ones(A.shape, dtype = dtype) * beta.value)).all():
        raise RuntimeError("Failed: CPU scalar vector init")
    print("Test: initialisation of vector with CPU scalar passed")

    # + ndarray
    X = p.Vector(np.ones(A.shape, dtype = dtype) * beta.value)
    if not (X == (np.ones(A.shape, dtype = dtype) * beta.value)).all():
        raise RuntimeError("Failed: ndarray vector init")
    print("Test: initialisation of vector with ndarray passed")

    # + Vector
    X = p.Vector(Y)
    if not (X == Y).all():
        raise RuntimeError("Failed: Vector Vector init")
    print("Test: initialisation of vector with Vector passed")

    # Negation
    X = -vcl_A
    Y = -vcl_A.value
    act_diff = math.fabs(diff(X, Y))
    if act_diff > epsilon:
        raise RuntimeError("Failed: negation")
    print("Test: negation passed")

    # Inner product
    X = vcl_A.dot(vcl_B)
    Y = vcl_A.value.dot(vcl_B.value)
    act_diff = math.fabs(X - Y)
    if act_diff > 0.01: # NB: numpy seems to be imprecise here
        raise RuntimeError("Failed: inner product of vectors")
    print("Test: inner product of vectors passed")

    # In-place scaling (multiplication by scalar)
    X = vcl_A.value
    X *= beta.value
    vcl_A *= beta
    act_diff = math.fabs(diff(X, vcl_A))
    if act_diff > epsilon:
        raise RuntimeError("Failed: in-place scale (multiplication)")
    print("Test: in-place scale (multiplication) passed")

    # In-place scaling (division by scalar)
    X = vcl_A.value
    X /= alpha.value
    vcl_A /= alpha
    act_diff = math.fabs(diff(X, vcl_A))
    if act_diff > epsilon:
        raise RuntimeError("Failed: in-place scale (division)")
    print("Test: in-place scale (division) passed")

    # In-place add
    X = vcl_A.value
    X += vcl_B.value
    vcl_A += vcl_B
    act_diff = math.fabs(diff(X, vcl_A))
    if act_diff > epsilon:
        raise RuntimeError("Failed: in-place add")
    print("Test: in-place add passed")

    # Scaled in-place add
    X += alpha.value * vcl_B.value
    vcl_A += alpha * vcl_B
    act_diff = math.fabs(diff(X, vcl_A))
    if act_diff > epsilon:
        raise RuntimeError("Failed: scaled in-place add")
    print("Test: scaled in-place add passed")

    # Add
    Y = vcl_A.value + vcl_B.value
    Z = vcl_A + vcl_B
    act_diff = math.fabs(diff(Y, Z))
    if act_diff > epsilon:
        raise RuntimeError("Failed: add")
    print("Test: add passed")

    # Scaled add (left)
    Y = dtype(alpha.value) * vcl_B.value + vcl_C.value
    Z = alpha * vcl_B + vcl_C
    act_diff = math.fabs(diff(Y, Z))
    if act_diff > epsilon:
        print(act_diff)
        print(Y, type(Y))
        print(Z, type(Z))
        print(Z - Y)
        raise RuntimeError("Failed: scaled add (left)")
    print("Test: scaled add (left) passed")

    # Scaled add (right)
    Y = vcl_B.value + dtype(alpha.value) * vcl_C.value
    Z = vcl_B + alpha * vcl_C
    act_diff = math.fabs(diff(Y, Z))
    if act_diff > epsilon: # (Z == Y).all():
        pass
        raise RuntimeError("Failed: scaled add (left)")
    print("Test: scaled add (right) passed")

    # Scaled add (both)
    Y = alpha.value * vcl_B.value + alpha.value * vcl_C.value
    Z = alpha * vcl_B + alpha * vcl_C
    act_diff = math.fabs(diff(Y, Z))
    if act_diff > epsilon:
        pass
        raise RuntimeError("Failed: scaled add (both)")
    print("Test: scaled add (both) passed")

    # In-place sub
    X = vcl_A.value
    X -= vcl_B.value
    vcl_A -= vcl_B
    if not (vcl_A == X).all():
        raise RuntimeError("Failed: in-place sub")
    print("Test: in-place sub passed")

    # Scaled in-place sub
    X -= alpha.value * vcl_B.value
    vcl_A -= alpha * vcl_B
    if not (vcl_A == X).all():
        raise RuntimeError("Failed: scaled in-place sub")
    print("Test: scaled in-place sub passed")

    # Sub
    Y = vcl_A.value - vcl_B.value
    Z = vcl_A - vcl_B
    if not (Y == Z).all():
        raise RuntimeError("Failed: sub")
    print("Test: sub passed")

    # Scaled sub (left)
    Y = alpha.value * vcl_B.value - vcl_C.value
    Z = alpha * vcl_B - vcl_C
    act_diff = math.fabs(diff(Y, Z))
    if act_diff > epsilon:
        raise RuntimeError("Failed: scaled sub (left)")
    print("Test: scaled sub (left) passed")

    # Scaled sub (right)
    Y = vcl_B.value - alpha.value * vcl_C.value
    Z = vcl_B - alpha * vcl_C
    act_diff = math.fabs(diff(Y, Z))
    if act_diff > epsilon:
        raise RuntimeError("Failed: scaled sub (right)")
    print("Test: scaled sub (right) passed")

    # Scaled sub (both)
    Y = alpha.value * vcl_B.value - alpha.value * vcl_C.value
    Z = alpha * vcl_B - alpha * vcl_C
    act_diff = math.fabs(diff(Y, Z))
    if act_diff > epsilon:
        raise RuntimeError("Failed: scaled sub (both)")
    print("Test: scaled sub (both) passed")

    # Scalar multiplication (CPU scalar) -- not supported yet
    #gamma_py = beta.value * beta.value
    #gamma_vcl = beta * beta
    # ...
    # Scalar multiplication (GPU scalar)

    # Matrix divided by scalar
    X = vcl_A.value / alpha.value
    Y = vcl_A / alpha
    act_diff = math.fabs(diff(X, Y))
    if act_diff > epsilon:
        raise RuntimeError("Failed: matrix-scalar division")
    print("Test: matrix-scalar division passed")

    # Binary elementwise operations -- prod and div
    X = vcl_A.value * vcl_B.value
    Y = p.ElementProd(vcl_A, vcl_B)
    act_diff = math.fabs(diff(X, Y))
    if act_diff > epsilon:
        raise RuntimeError("Failed: elementwise matrix-matrix multiplication")
    print("Test: elementwise matrix-matrix multiplication passed")

    X = vcl_A.value / vcl_B.value
    Y = p.ElementDiv(vcl_A, vcl_B)
    act_diff = math.fabs(diff(X, Y))
    if act_diff > epsilon:
        raise RuntimeError("Failed: elementwise matrix-matrix division")
    print("Test: elementwise matrix-matrix division passed")

    # Unary elementwise operations
    # - abs TODO
    #X = abs(vcl_A.value)
    #Y = p.ElementAbs(vcl_A)
    #act_diff = math.fabs(diff(X, Y))
    #if act_diff > epsilon:
    #    raise RuntimeError("Failed: elementwise abs")
    #print("Test: elementwise abs passed")

    # - acos
    X = np.arccos(vcl_A.value)
    Y = p.ElementAcos(vcl_A).result # TODO THIS SHOULDN'T BE REQUIRED
    act_diff = math.fabs(diff(X, Y))
    if act_diff > epsilon:
        raise RuntimeError("Failed: elementwise acos")
    print("Test: elementwise acos passed")

    # - asin
    X = np.arcsin(vcl_A.value)
    Y = p.ElementAsin(vcl_A).result
    act_diff = math.fabs(diff(X, Y))
    if act_diff > epsilon:
        raise RuntimeError("Failed: elementwise asin")
    print("Test: elementwise asin passed")

    # - atan
    X = np.arctan(vcl_A.value)
    Y = p.ElementAtan(vcl_A).result
    act_diff = math.fabs(diff(X, Y))
    if act_diff > epsilon:
        raise RuntimeError("Failed: elementwise atan")
    print("Test: elementwise atan passed")

    # - ceil
    X = np.ceil(vcl_A.value)
    Y = p.ElementCeil(vcl_A).result
    act_diff = math.fabs(diff(X, Y))
    if act_diff > epsilon:
        raise RuntimeError("Failed: elementwise ceil")
    print("Test: elementwise ceil passed")

    # - cos
    X = np.cos(vcl_A.value)
    Y = p.ElementCos(vcl_A).result
    act_diff = math.fabs(diff(X, Y))
    if act_diff > epsilon:
        raise RuntimeError("Failed: elementwise cos")
    print("Test: elementwise cos passed")

    # - cosh
    X = np.cosh(vcl_A.value)
    Y = p.ElementCosh(vcl_A).result
    act_diff = math.fabs(diff(X, Y))
    if act_diff > epsilon:
        raise RuntimeError("Failed: elementwise cosh")
    print("Test: elementwise cosh passed")

    # - exp
    X = np.exp(vcl_A.value)
    Y = p.ElementExp(vcl_A).result
    act_diff = math.fabs(diff(X, Y))
    if act_diff > epsilon:
        raise RuntimeError("Failed: elementwise exp")
    print("Test: elementwise exp passed")

    # - fabs
    X = np.fabs(vcl_A.value)
    Y = p.ElementFabs(vcl_A).result
    act_diff = math.fabs(diff(X, Y))
    if act_diff > epsilon:
        raise RuntimeError("Failed: elementwise fabs")
    print("Test: elementwise fabs passed")

    # - floor
    X = np.floor(vcl_A.value)
    Y = p.ElementFloor(vcl_A).result
    act_diff = math.fabs(diff(X, Y))
    if act_diff > epsilon:
        raise RuntimeError("Failed: elementwise floor")
    print("Test: elementwise floor passed")

    # - log
    X = np.log(vcl_A.value)
    Y = p.ElementLog(vcl_A).result
    act_diff = math.fabs(diff(X, Y))
    if act_diff > epsilon:
        raise RuntimeError("Failed: elementwise log")
    print("Test: elementwise log passed")

    # - log10
    X = np.log10(vcl_A.value)
    Y = p.ElementLog10(vcl_A).result
    act_diff = math.fabs(diff(X, Y))
    if act_diff > epsilon:
        raise RuntimeError("Failed: elementwise log10")
    print("Test: elementwise log10 passed")

    # - sin
    X = np.sin(vcl_A.value)
    Y = p.ElementSin(vcl_A).result
    act_diff = math.fabs(diff(X, Y))
    if act_diff > epsilon:
        raise RuntimeError("Failed: elementwise sin")
    print("Test: elementwise sin passed")

    # - sinh
    X = np.sinh(vcl_A.value)
    Y = p.ElementSinh(vcl_A).result
    act_diff = math.fabs(diff(X, Y))
    if act_diff > epsilon:
        raise RuntimeError("Failed: elementwise sinh")
    print("Test: elementwise sinh passed")

    # - sqrt
    X = np.sqrt(vcl_A.value)
    Y = p.ElementSqrt(vcl_A).result
    act_diff = math.fabs(diff(X, Y))
    if act_diff > epsilon:
        raise RuntimeError("Failed: elementwise sqrt")
    print("Test: elementwise sqrt passed")

    # - tan
    X = np.tan(vcl_A.value)
    Y = p.ElementTan(vcl_A).result
    act_diff = math.fabs(diff(X, Y))
    if act_diff > epsilon:
        raise RuntimeError("Failed: elementwise tan")
    print("Test: elementwise tan passed")

    # - tanh
    X = np.tanh(vcl_A.value)
    Y = p.ElementTanh(vcl_A).result
    act_diff = math.fabs(diff(X, Y))
    if act_diff > epsilon:
        raise RuntimeError("Failed: elementwise tanh")
    print("Test: elementwise tanh passed")

    # - norm1
    X = np.linalg.norm(vcl_A.value, 1)
    Y = p.norm(vcl_A, 1) # or vcl_A.norm(1)
    act_diff = math.fabs(X - Y)
    if act_diff > epsilon:
        print(vcl_A)
        #raise RuntimeError("Failed: norm(1)")
    print("Test: norm(1) passed")

    # - norm2
    X = np.linalg.norm(vcl_A.value, 2)
    Y = vcl_A.norm(2) # or vcl_A.norm(1)
    act_diff = math.fabs(X - Y)
    if act_diff > epsilon:
        raise RuntimeError("Failed: norm(2)")
    print("Test: norm(2) passed")

    # - norm_inf
    X = np.linalg.norm(vcl_A.value, np.inf)
    Y = vcl_A.norm(np.inf)
    act_diff = math.fabs(X - Y)
    if act_diff > epsilon:
        raise RuntimeError("Failed: norm(inf)")
    print("Test: norm(inf) passed")

    # in-place multiply-division-add
    X = vcl_C.value
    X += alpha.value * vcl_A.value + vcl_B.value / beta.value
    vcl_C += alpha * vcl_A + vcl_B / beta
    act_diff = math.fabs(diff(X, vcl_C))
    if act_diff > epsilon:
        raise RuntimeError("Failed: in-place multiply-division-add")
    print("Test: in-place multiply-division-add passed")

    # lengthy sum of scaled vectors
    X = alpha.value * vcl_A.value - vcl_B.value / beta.value + vcl_A.value * beta.value - vcl_B.value / alpha.value + vcl_C.value
    Y = alpha * vcl_A - vcl_B / beta + vcl_A * beta - vcl_B / alpha + vcl_C
    act_diff = math.fabs(diff(X, Y))
    if act_diff > epsilon:
        raise RuntimeError("Failed: lengthy sum of scaled vectors")
    print("Test: lengthy sum of scaled vectors passed")

    # sub-expression
    X = vcl_A.value + (((vcl_C.value + vcl_B.value) * alpha.value) - vcl_B.value) / beta.value
    Y = vcl_A + (((vcl_C + vcl_B) * alpha) - vcl_B) / beta
    act_diff = math.fabs(diff(X, Y))
    if act_diff > epsilon:
        raise RuntimeError("Failed: vector sub-expression test %s")
    print("Test: vector sub-expression passed")

    # plane rotation
    V = (alpha * vcl_A + beta * vcl_B).result
    W = (alpha * vcl_B - beta * vcl_A).result
    p.plane_rotation(vcl_A, vcl_B, alpha, beta)
    act_diffB = math.fabs(diff(W, vcl_B))
    act_diffA = math.fabs(diff(V, vcl_A))
    act_diffA = math.fabs(diff(V.value, vcl_A.value))
    if act_diffA > epsilon or act_diffB > epsilon:
        print(act_diffA, act_diffB)
        print(vcl_A)
        print(V)
        print(p.ElementFabs(V - vcl_A))
        #print(W, vcl_B)
        raise RuntimeError("Failed: plane rotation")
    print("Test: plane rotation passed")

    return os.EX_OK


def test():
    print("----------------------------------------------")
    print("----------------------------------------------")
    print("## Test :: vector operations") #TODO
    print("----------------------------------------------")
    print("----------------------------------------------")

    print("*** Using float numeric type ***")
    print("# Testing setup:")
    epsilon = 1.0E-3
    print("  eps:      %s" % epsilon)
    test_vector_slice(run_test, epsilon, p.float32, 11)

    print("*** Using double numeric type ***")
    print("# Testing setup:")
    epsilon = 1.0E-11
    print("  eps:      %s" % epsilon)
    test_vector_slice(run_test, epsilon, p.float64, 11)

    print("# Test passed")
    
    return os.EX_OK


if __name__ == "__main__":
    sys.exit(test())




import pyviennacl as p

def read_mtx(fname, dtype):
    """
    Read a MatrixMarket file. Assume coordinate format, and double precision.
    
    Very crude!
    """
    fd = open(fname)
    lines = map(lambda x: x.strip().split(" "), fd.readlines())
    ln = -1
    for line in lines:
        ln += 1
        if line[ln][0] == "%":
            continue
        else:
            break
    n = int(lines[ln][0])
    m = int(lines[ln][1])
    nnz = int(lines[ln][2])
    mat = p.CompressedMatrix(n, m, nnz, dtype=dtype)
    mat_type = p.np_result_type(mat).type
    def assign(l):
        try:
            i, j, v = int(l[0]), int(l[1]), mat_type(l[2])
            mat[i-1, j-1] = v
        except ValueError:
            pass
    map(assign, lines[ln+1:])
    return mat

def read_vector(fname, dtype):
    fd = open(fname)
    lines = map(lambda x: x.strip().split(" "), fd.readlines())
    count = int(lines[0][0])
    vector = list(map(lambda x: p.np_result_type(dtype).type(x), lines[1]))
    vector = p.Vector(vector, dtype = dtype)
    if vector.size != count:
        raise Exception("Sizes %d and %d do not match!" % (vector.size, count))
    return vector
    



#!/usr/bin/env python

import math
import numpy as np
import os
import pyviennacl as p
import random

import logging
logger = logging.getLogger('pyviennacl')

def noop(*args, **kwargs):
    return os.EX_OK

def diff(a, b):
    p.util.backend_finish()
    ret = 0

    # Convert NumPy types to ViennaCL types (they're much more convenient!)
    if isinstance(a, np.ndarray):
        if a.ndim == 1:
            a = p.Vector(a, dtype = a.dtype)
        elif a.ndim == 2:
            if isinstance(b, p.MagicMethods):
                a = p.Matrix(a, dtype = a.dtype, layout = b.layout)
            else:
                a = p.Matrix(a, dtype = a.dtype)
        else:
            raise TypeError("Something went wrong")

    if isinstance(b, np.ndarray):
        if b.ndim == 1:
            b = p.Vector(b)
        elif b.ndim == 2:
            if isinstance(a, p.MagicMethods):
                b = p.Matrix(b, dtype = b.dtype, layout = a.layout)
            else:
                b = p.Matrix(b, dtype = b.dtype)
        else:
            raise TypeError("Something went wrong")

    # The MagicMethods class guarantees that we have some useful facilities
    #   (and both Node and Leaf are derived from MagicMethods)
    if isinstance(a, p.MagicMethods) and isinstance(b, p.MagicMethods):
        if a.layout != b.layout:
            # We want to make sure that a and b have the same layout
            # So construct a temporary matrix, and assign b to it
            temp = p.Matrix(b.shape, dtype = b.dtype, layout = a.layout)
            p.Assign(temp, b.result).execute()
            b = temp
            log.warning("HERE")
        d = p.ElementFabs(a - b)
        cpu_d = d.as_ndarray()
        if len(d.shape) == 1:
            # vector
            for i in range(d.shape[0]):
                act = math.fabs(cpu_d[i])
                if act > ret:
                    ret = act
        elif len(d.shape) == 2:
            # matrix
            for i in range(d.shape[0]):
                for j in range(d.shape[1]): 
                    act = math.fabs(cpu_d[i, j])
                    if act > ret:
                        ret = act
        else:
            raise TypeError("Something went wrong..")
        return ret
    else:
        # We don't have either ndarrays or ViennaCL types so assume plain scalar
        return math.fabs(a - b) / max(math.fabs(a), math.fabs(b))


def test_vector_slice(test_func,
                      epsilon, dtype,
                      size = 131):
    # Create reference numpy types
    A = np.empty((size,), dtype = dtype)
    big_A = np.ones((size*4,), dtype = dtype) * 3.1415
    B = np.empty((size,), dtype = dtype)
    big_B = np.ones((size*4,), dtype = dtype) * 42.0
    C = np.empty((size,), dtype = dtype)
    big_C = np.ones((size*4,), dtype = dtype) * 2.718

    # Fill A and B with random values
    for i in range(A.shape[0]):
        A[i] = random.random()
    for i in range(B.shape[0]):
        B[i] = random.random()

    # Construct appropriate ViennaCL objects
    vcl_A = p.Vector(A)

    vcl_big_range_A = p.Vector(big_A)
    vcl_big_range_A[size:2*size] = vcl_A
    vcl_range_A = vcl_big_range_A[size:2*size]

    vcl_big_slice_A = p.Vector(big_A)
    vcl_big_slice_A[size:-size:2] = vcl_A
    vcl_slice_A = vcl_big_slice_A[size:-size:2]

    vcl_B = p.Vector(B)

    vcl_big_range_B = p.Vector(big_B)
    vcl_big_range_B[size:2*size] = vcl_B
    vcl_range_B = vcl_big_range_B[size:2*size]

    vcl_big_slice_B = p.Vector(big_B)
    vcl_big_slice_B[size:-size:2] = vcl_B
    vcl_slice_B = vcl_big_slice_B[size:-size:2]

    vcl_C = p.Vector(C)

    vcl_big_range_C = p.Vector(big_C)
    vcl_range_C = vcl_big_range_C[(size - 1):(2*size - 1)]

    vcl_big_slice_C = p.Vector(big_C)
    vcl_slice_C = vcl_big_slice_C[(size - 1):(4*size - 1):3]

    # A=vector, B=vector, C=vector
    print("Now using A=vector, B=vector, C=vector")
    try:
        ret = test_func(epsilon,
                        A, B, C,
                        vcl_A, vcl_B, vcl_C,
                        dtype = dtype)
    except TypeError as e:
        if not "Vectors do not have the same layout" in e.args[0]:
            raise
        else:
            logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=vector, B=vector, C=range
    print("Now using A=vector, B=vector, C=range")
    try:
        ret = test_func(epsilon,
                        A, B, C,
                        vcl_A, vcl_B, vcl_range_C,
                        dtype = dtype)
    except TypeError as e:
        if not "Vectors do not have the same layout" in e.args[0]:
            raise
        else:
            logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=vector, B=vector, C=slice
    #print("Now using A=vector, B=vector, C=slice")
    #try:
    #    ret = test_func(epsilon,
    #                    A, B, C,
    #                    vcl_A, vcl_B, vcl_slice_C,
    #                    dtype = dtype)
    #except TypeError as e:
    #    if not "Vectors do not have the same layout" in e.args[0]:
    #        raise
    #    else:
    #        logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=vector, B=range, C=vector
    print("Now using A=vector, B=range, C=vector")
    try:
        ret = test_func(epsilon,
                        A, B, C,
                        vcl_A, vcl_range_B, vcl_C,
                        dtype = dtype)
    except TypeError as e:
        if not "Vectors do not have the same layout" in e.args[0]:
            raise
        else:
            logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=vector, B=range, C=range
    print("Now using A=vector, B=range, C=range")
    try:
        ret = test_func(epsilon,
                        A, B, C,
                        vcl_A, vcl_range_B, vcl_range_C,
                        dtype = dtype)
    except TypeError as e:
        if not "Vectors do not have the same layout" in e.args[0]:
            raise
        else:
            logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=vector, B=range, C=slice
    #print("Now using A=vector, B=range, C=slice")
    #try:
    #    ret = test_func(epsilon,
    #                    A, B, C,
    #                    vcl_A, vcl_range_B, vcl_slice_C,
    #                    dtype = dtype)
    #except TypeError as e:
    #    if not "Vectors do not have the same layout" in e.args[0]:
    #        raise
    #    else:
    #        logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=vector, B=slice, C=vector
    #print("Now using A=vector, B=slice, C=vector")
    #try:
    #    ret = test_func(epsilon,
    #                    A, B, C,
    #                    vcl_A, vcl_slice_B, vcl_C,
    #                    dtype = dtype)
    #except TypeError as e:
    #    if not "Vectors do not have the same layout" in e.args[0]:
    #        raise
    #    else:
    #        logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=vector, B=slice, C=range
    #print("Now using A=vector, B=slice, C=range")
    #try:
    #    ret = test_func(epsilon,
    #                    A, B, C,
    #                    vcl_A, vcl_slice_B, vcl_range_C,
    #                    dtype = dtype)
    #except TypeError as e:
    #    if not "Vectors do not have the same layout" in e.args[0]:
    #        raise
    #    else:
    #        logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=vector, B=slice, C=slice
    #print("Now using A=vector, B=slice, C=slice")
    #try:
    #    ret = test_func(epsilon,
    #                    A, B, C,
    #                    vcl_A, vcl_slice_B, vcl_slice_C,
    #                    dtype = dtype)
    #except TypeError as e:
    #    if not "Vectors do not have the same layout" in e.args[0]:
    #        raise
    #    else:
    #        logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=range, B=vector, C=vector
    print("Now using A=range, B=vector, C=vector")
    try:
        ret = test_func(epsilon,
                        A, B, C,
                        vcl_range_A, vcl_B, vcl_C,
                        dtype = dtype)
    except TypeError as e:
        if not "Vectors do not have the same layout" in e.args[0]:
            raise
        else:
            logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=range, B=vector, C=range
    print("Now using A=range, B=vector, C=range")
    try:
        ret = test_func(epsilon,
                        A, B, C,
                        vcl_range_A, vcl_B, vcl_range_C,
                        dtype = dtype)
    except TypeError as e:
        if not "Vectors do not have the same layout" in e.args[0]:
            raise
        else:
            logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=range, B=vector, C=slice
    #print("Now using A=range, B=vector, C=slice")
    #try:
    #    ret = test_func(epsilon,
    #                    A, B, C,
    #                    vcl_range_A, vcl_B, vcl_slice_C,
    #                    dtype = dtype)
    #except TypeError as e:
    #    if not "Vectors do not have the same layout" in e.args[0]:
    #        raise
    #    else:
    #        logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=range, B=range, C=vector
    print("Now using A=range, B=range, C=vector")
    try:
        ret = test_func(epsilon,
                        A, B, C,
                        vcl_range_A, vcl_range_B, vcl_C,
                        dtype = dtype)
    except TypeError as e:
        if not "Vectors do not have the same layout" in e.args[0]:
            raise
        else:
            logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=range, B=range, C=range
    print("Now using A=range, B=range, C=range")
    try:
        ret = test_func(epsilon,
                        A, B, C,
                        vcl_range_A, vcl_range_B, vcl_range_C,
                        dtype = dtype)
    except TypeError as e:
        if not "Vectors do not have the same layout" in e.args[0]:
            raise
        else:
            logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=range, B=range, C=slice
    #print("Now using A=range, B=range, C=slice")
    #try:
    #    ret = test_func(epsilon,
    #                    A, B, C,
    #                    vcl_range_A, vcl_range_B, vcl_slice_C,
    #                    dtype = dtype)
    #except TypeError as e:
    #    if not "Vectors do not have the same layout" in e.args[0]:
    #        raise
    #    else:
    #        logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=range, B=slice, C=vector
    #print("Now using A=range, B=slice, C=vector")
    #try:
    #    ret = test_func(epsilon,
    #                    A, B, C,
    #                    vcl_range_A, vcl_slice_B, vcl_C,
    #                    dtype = dtype)
    #except TypeError as e:
    #    if not "Vectors do not have the same layout" in e.args[0]:
    #        raise
    #    else:
    #        logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=range, B=slice, C=range
    #print("Now using A=range, B=slice, C=range")
    #try:
    #    ret = test_func(epsilon,
    #                    A, B, C,
    #                    vcl_range_A, vcl_slice_B, vcl_range_C,
    #                    dtype = dtype)
    #except TypeError as e:
    #    if not "Vectors do not have the same layout" in e.args[0]:
    #        raise
    #    else:
    #        logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=range, B=slice, C=slice
    #print("Now using A=range, B=slice, C=slice")
    #try:
    #    ret = test_func(epsilon,
    #                    A, B, C,
    #                    vcl_range_A, vcl_slice_B, vcl_slice_C,
    #                    dtype = dtype)
    #except TypeError as e:
    #    if not "Vectors do not have the same layout" in e.args[0]:
    #        raise
    #    else:
    #        logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=slice, B=vector, C=vector
    #print("Now using A=slice, B=vector, C=vector")
    #try:
    #    ret = test_func(epsilon,
    #                    A, B, C,
    #                    vcl_slice_A, vcl_B, vcl_C,
    #                    dtype = dtype)
    #except TypeError as e:
    #    if not "Vectors do not have the same layout" in e.args[0]:
    #        raise
    #    else:
    #        logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=slice, B=vector, C=range
    #print("Now using A=slice, B=vector, C=range")
    #try:
    #    ret = test_func(epsilon,
    #                    A, B, C,
    #                    vcl_slice_A, vcl_B, vcl_range_C,
    #                    dtype = dtype)
    #except TypeError as e:
    #    if not "Vectors do not have the same layout" in e.args[0]:
    #        raise
    #    else:
    #        logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=slice, B=vector, C=slice
    #print("Now using A=slice, B=vector, C=slice")
    #try:
    #    ret = test_func(epsilon,
    #                    A, B, C,
    #                    vcl_slice_A, vcl_B, vcl_slice_C,
    #                    dtype = dtype)
    #except TypeError as e:
    #    if not "Vectors do not have the same layout" in e.args[0]:
    #        raise
    #    else:
    #        logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=slice, B=range, C=vector
    #print("Now using A=slice, B=range, C=vector")
    #try:
    #    ret = test_func(epsilon,
    #                    A, B, C,
    #                    vcl_slice_A, vcl_range_B, vcl_C,
    #                    dtype = dtype)
    #except TypeError as e:
    #    if not "Vectors do not have the same layout" in e.args[0]:
    #        raise
    #    else:
    #        logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=slice, B=range, C=range
    #print("Now using A=slice, B=range, C=range")
    #try:
    #    ret = test_func(epsilon,
    #                    A, B, C,
    #                    vcl_slice_A, vcl_range_B, vcl_range_C,
    #                    dtype = dtype)
    #except TypeError as e:
    #    if not "Vectors do not have the same layout" in e.args[0]:
    #        raise
    #    else:
    #        logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=slice, B=range, C=slice
    #print("Now using A=slice, B=range, C=slice")
    #try:
    #    ret = test_func(epsilon,
    #                    A, B, C,
    #                    vcl_slice_A, vcl_range_B, vcl_slice_C,
    #                    dtype = dtype)
    #except TypeError as e:
    #    if not "Vectors do not have the same layout" in e.args[0]:
    #        raise
    #    else:
    #        logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=slice, B=slice, C=vector
    #print("Now using A=slice, B=slice, C=vector")
    #try:
    #    ret = test_func(epsilon,
    #                    A, B, C,
    #                    vcl_slice_A, vcl_slice_B, vcl_C,
    #                    dtype = dtype)
    #except TypeError as e:
    #    if not "Vectors do not have the same layout" in e.args[0]:
    #        raise
    #    else:
    #        logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=slice, B=slice, C=range
    #print("Now using A=slice, B=slice, C=range")
    #try:
    #    ret = test_func(epsilon,
    #                    A, B, C,
    #                    vcl_slice_A, vcl_slice_B, vcl_range_C,
    #                    dtype = dtype)
    #except TypeError as e:
    #    if not "Vectors do not have the same layout" in e.args[0]:
    #        raise
    #    else:
    #        logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=slice, B=slice, C=slice
    #print("Now using A=slice, B=slice, C=slice")
    #try:
    #    ret = test_func(epsilon,
    #                    A, B, C,
    #                    vcl_slice_A, vcl_slice_B, vcl_slice_C,
    #                    dtype = dtype)
    #except TypeError as e:
    #    if not "Vectors do not have the same layout" in e.args[0]:
    #        raise
    #    else:
    #        logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    return os.EX_OK


def test_matrix_slice(test_func,
                      epsilon, dtype,
                      A_layout = p.ROW_MAJOR, B_layout = p.ROW_MAJOR, 
                      C_layout = p.ROW_MAJOR,
                      size1 = 131, size2 = 67, size3 = 73):
    if A_layout == p.ROW_MAJOR:
        A_order = 'C'
    else:
        A_order = 'F'

    if B_layout == p.ROW_MAJOR:
        B_order = 'C'
    else:
        B_order = 'F'

    if C_layout == p.ROW_MAJOR:
        C_order = 'C'
    else:
        C_order = 'F'

    # Create reference numpy types
    A = np.empty((size1, size2), dtype = dtype, order = A_order)
    big_A = np.ones((size1 * 4, size2 * 4), dtype = dtype, order = A_order) * 3.1415
    B = np.empty((size2, size3), dtype = dtype, order = B_order)
    big_B = np.ones((size2 * 4, size3 * 4), dtype = dtype, order = B_order) * 42.0
    C = np.empty((size1, size3), dtype = dtype, order = C_order)
    big_C = np.ones((size1 * 4, size3 * 4), dtype = dtype, order = C_order) * 2.718

    # Fill A and B with random values
    for i in range(A.shape[0]):
        for j in range(A.shape[1]):
            A[i, j] = random.random()
    for i in range(B.shape[0]):
        for j in range(B.shape[1]):
            B[i, j] = random.random()

    A_trans = A.T
    big_A_trans = big_A.T
    B_trans = B.T
    big_B_trans = big_B.T
    C_trans = C.T
    big_C_trans = big_C.T

    # Construct appropriate ViennaCL objects
    vcl_A = p.Matrix(A, layout = A_layout)

    vcl_big_range_A = p.Matrix(big_A, layout = A_layout)
    vcl_big_range_A[size1:2*size1, size2:2*size2] = vcl_A
    vcl_range_A = vcl_big_range_A[size1:2*size1, size2:2*size2]

    vcl_big_slice_A = p.Matrix(big_A, layout = A_layout)
    vcl_big_slice_A[size1:-size1:2, size2::3] = vcl_A
    vcl_slice_A = vcl_big_slice_A[size1:-size1:2, size2::3]

    vcl_A_trans = p.Matrix(A_trans, layout = A_layout)

    vcl_big_range_A_trans = p.Matrix(big_A_trans, layout = A_layout)
    vcl_big_range_A_trans[size2:2*size2, size1:2*size1] = vcl_A_trans
    vcl_range_A_trans = vcl_big_range_A_trans[size2:2*size2, size1:2*size1]

    vcl_big_slice_A_trans = p.Matrix(big_A_trans, layout = A_layout)
    vcl_big_slice_A_trans[size2:-size2:2, size1::3] = vcl_A_trans
    vcl_slice_A_trans = vcl_big_slice_A_trans[size2:-size2:2, size1::3]

    vcl_B = p.Matrix(B, layout = B_layout)

    vcl_big_range_B = p.Matrix(big_B, layout = B_layout)
    vcl_big_range_B[size2:2*size2, size3:2*size3] = vcl_B
    vcl_range_B = vcl_big_range_B[size2:2*size2, size3:2*size3]

    vcl_big_slice_B = p.Matrix(big_B, layout = B_layout)
    vcl_big_slice_B[size2:-size2:2, size3::3] = vcl_B
    vcl_slice_B = vcl_big_slice_B[size2:-size2:2, size3::3]

    vcl_B_trans = p.Matrix(B_trans, layout = B_layout)

    vcl_big_range_B_trans = p.Matrix(big_B_trans, layout = B_layout)
    vcl_big_range_B_trans[size3:2*size3, size2:2*size2] = vcl_B_trans
    vcl_range_B_trans = vcl_big_range_B_trans[size3:2*size3, size2:2*size2]

    vcl_big_slice_B_trans = p.Matrix(big_B_trans, layout = B_layout)
    vcl_big_slice_B_trans[size3:-size3:2, size2::3] = vcl_B_trans
    vcl_slice_B_trans = vcl_big_slice_B_trans[size3:-size3:2, size2::3]

    vcl_C = p.Matrix(C, layout = C_layout)

    vcl_big_range_C = p.Matrix(big_C, layout = C_layout)
    vcl_range_C = vcl_big_range_C[(size1 - 1):(2*size1 - 1), (size3 - 1):(2*size3 - 1)]

    vcl_big_slice_C = p.Matrix(big_C, layout = C_layout)
    vcl_slice_C = vcl_big_slice_C[(size1 - 1):(4*size1 - 1):3, (size3 - 1):(4*size3 - 1):3]

    # A=matrix, B=matrix, C=matrix
    print("Now using A=matrix, B=matrix, C=matrix")
    try:
        ret = test_func(epsilon,
                        A, A_trans, B, B_trans, C,
                        vcl_A, vcl_A_trans,
                        vcl_B, vcl_B_trans, vcl_C,
                        dtype = dtype)
    except TypeError as e:
        if not "Matrices do not have the same layout" in e.args[0]:
            raise
        else:
            logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=matrix, B=matrix, C=range
    print("Now using A=matrix, B=matrix, C=range")
    try:
        ret = test_func(epsilon,
                        A, A_trans, B, B_trans, C,
                        vcl_A, vcl_A_trans,
                        vcl_B, vcl_B_trans, vcl_range_C,
                        dtype = dtype)
    except TypeError as e:
        if not "Matrices do not have the same layout" in e.args[0]:
            raise
        else:
            logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=matrix, B=matrix, C=slice
    print("Now using A=matrix, B=matrix, C=slice")
    try:
        ret = test_func(epsilon,
                        A, A_trans, B, B_trans, C,
                        vcl_A, vcl_A_trans,
                        vcl_B, vcl_B_trans, vcl_slice_C,
                        dtype = dtype)
    except TypeError as e:
        if not "Matrices do not have the same layout" in e.args[0]:
            raise
        else:
            logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=matrix, B=range, C=matrix
    print("Now using A=matrix, B=range, C=matrix")
    try:
        ret = test_func(epsilon,
                        A, A_trans, B, B_trans, C,
                        vcl_A, vcl_A_trans,
                        vcl_range_B, vcl_range_B_trans, vcl_C,
                        dtype = dtype)
    except TypeError as e:
        if not "Matrices do not have the same layout" in e.args[0]:
            raise
        else:
            logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=matrix, B=range, C=range
    print("Now using A=matrix, B=range, C=range")
    try:
        ret = test_func(epsilon,
                        A, A_trans, B, B_trans, C,
                        vcl_A, vcl_A_trans,
                        vcl_range_B, vcl_range_B_trans, vcl_range_C,
                        dtype = dtype)
    except TypeError as e:
        if not "Matrices do not have the same layout" in e.args[0]:
            raise
        else:
            logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=matrix, B=range, C=slice
    print("Now using A=matrix, B=range, C=slice")
    try:
        ret = test_func(epsilon,
                        A, A_trans, B, B_trans, C,
                        vcl_A, vcl_A_trans,
                        vcl_range_B, vcl_range_B_trans, vcl_slice_C,
                        dtype = dtype)
    except TypeError as e:
        if not "Matrices do not have the same layout" in e.args[0]:
            raise
        else:
            logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=matrix, B=slice, C=matrix
    print("Now using A=matrix, B=slice, C=matrix")
    try:
        ret = test_func(epsilon,
                        A, A_trans, B, B_trans, C,
                        vcl_A, vcl_A_trans,
                        vcl_slice_B, vcl_slice_B_trans, vcl_C,
                        dtype = dtype)
    except TypeError as e:
        if not "Matrices do not have the same layout" in e.args[0]:
            raise
        else:
            logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=matrix, B=slice, C=range
    print("Now using A=matrix, B=slice, C=range")
    try:
        ret = test_func(epsilon,
                        A, A_trans, B, B_trans, C,
                        vcl_A, vcl_A_trans,
                        vcl_slice_B, vcl_slice_B_trans, vcl_range_C,
                        dtype = dtype)
    except TypeError as e:
        if not "Matrices do not have the same layout" in e.args[0]:
            raise
        else:
            logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=matrix, B=slice, C=slice
    print("Now using A=matrix, B=slice, C=slice")
    try:
        ret = test_func(epsilon,
                        A, A_trans, B, B_trans, C,
                        vcl_A, vcl_A_trans,
                        vcl_slice_B, vcl_slice_B_trans, vcl_slice_C,
                        dtype = dtype)
    except TypeError as e:
        if not "Matrices do not have the same layout" in e.args[0]:
            raise
        else:
            logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=range, B=matrix, C=matrix
    print("Now using A=range, B=matrix, C=matrix")
    try:
        ret = test_func(epsilon,
                        A, A_trans, B, B_trans, C,
                        vcl_range_A, vcl_range_A_trans,
                        vcl_B, vcl_B_trans, vcl_C,
                        dtype = dtype)
    except TypeError as e:
        if not "Matrices do not have the same layout" in e.args[0]:
            raise
        else:
            logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=range, B=matrix, C=range
    print("Now using A=range, B=matrix, C=range")
    try:
        ret = test_func(epsilon,
                        A, A_trans, B, B_trans, C,
                        vcl_range_A, vcl_range_A_trans,
                        vcl_B, vcl_B_trans, vcl_range_C,
                        dtype = dtype)
    except TypeError as e:
        if not "Matrices do not have the same layout" in e.args[0]:
            raise
        else:
            logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=range, B=matrix, C=slice
    print("Now using A=range, B=matrix, C=slice")
    try:
        ret = test_func(epsilon,
                        A, A_trans, B, B_trans, C,
                        vcl_range_A, vcl_range_A_trans,
                        vcl_B, vcl_B_trans, vcl_slice_C,
                        dtype = dtype)
    except TypeError as e:
        if not "Matrices do not have the same layout" in e.args[0]:
            raise
        else:
            logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=range, B=range, C=matrix
    print("Now using A=range, B=range, C=matrix")
    try:
        ret = test_func(epsilon,
                        A, A_trans, B, B_trans, C,
                        vcl_range_A, vcl_range_A_trans,
                        vcl_range_B, vcl_range_B_trans, vcl_C,
                        dtype = dtype)
    except TypeError as e:
        if not "Matrices do not have the same layout" in e.args[0]:
            raise
        else:
            logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=range, B=range, C=range
    print("Now using A=range, B=range, C=range")
    try:
        ret = test_func(epsilon,
                        A, A_trans, B, B_trans, C,
                        vcl_range_A, vcl_range_A_trans,
                        vcl_range_B, vcl_range_B_trans, vcl_range_C,
                        dtype = dtype)
    except TypeError as e:
        if not "Matrices do not have the same layout" in e.args[0]:
            raise
        else:
            logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=range, B=range, C=slice
    print("Now using A=range, B=range, C=slice")
    try:
        ret = test_func(epsilon,
                        A, A_trans, B, B_trans, C,
                        vcl_range_A, vcl_range_A_trans,
                        vcl_range_B, vcl_range_B_trans, vcl_slice_C,
                        dtype = dtype)
    except TypeError as e:
        if not "Matrices do not have the same layout" in e.args[0]:
            raise
        else:
            logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=range, B=slice, C=matrix
    print("Now using A=range, B=slice, C=matrix")
    try:
        ret = test_func(epsilon,
                        A, A_trans, B, B_trans, C,
                        vcl_range_A, vcl_range_A_trans,
                        vcl_slice_B, vcl_slice_B_trans, vcl_C,
                        dtype = dtype)
    except TypeError as e:
        if not "Matrices do not have the same layout" in e.args[0]:
            raise
        else:
            logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=range, B=slice, C=range
    print("Now using A=range, B=slice, C=range")
    try:
        ret = test_func(epsilon,
                        A, A_trans, B, B_trans, C,
                        vcl_range_A, vcl_range_A_trans,
                        vcl_slice_B, vcl_slice_B_trans, vcl_range_C,
                        dtype = dtype)
    except TypeError as e:
        if not "Matrices do not have the same layout" in e.args[0]:
            raise
        else:
            logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=range, B=slice, C=slice
    print("Now using A=range, B=slice, C=slice")
    try:
        ret = test_func(epsilon,
                        A, A_trans, B, B_trans, C,
                        vcl_range_A, vcl_range_A_trans,
                        vcl_slice_B, vcl_slice_B_trans, vcl_slice_C,
                        dtype = dtype)
    except TypeError as e:
        if not "Matrices do not have the same layout" in e.args[0]:
            raise
        else:
            logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=slice, B=matrix, C=matrix
    print("Now using A=slice, B=matrix, C=matrix")
    try:
        ret = test_func(epsilon,
                        A, A_trans, B, B_trans, C,
                        vcl_slice_A, vcl_slice_A_trans,
                        vcl_B, vcl_B_trans, vcl_C,
                        dtype = dtype)
    except TypeError as e:
        if not "Matrices do not have the same layout" in e.args[0]:
            raise
        else:
            logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=slice, B=matrix, C=range
    print("Now using A=slice, B=matrix, C=range")
    try:
        ret = test_func(epsilon,
                        A, A_trans, B, B_trans, C,
                        vcl_slice_A, vcl_slice_A_trans,
                        vcl_B, vcl_B_trans, vcl_range_C,
                        dtype = dtype)
    except TypeError as e:
        if not "Matrices do not have the same layout" in e.args[0]:
            raise
        else:
            logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=slice, B=matrix, C=slice
    print("Now using A=slice, B=matrix, C=slice")
    try:
        ret = test_func(epsilon,
                        A, A_trans, B, B_trans, C,
                        vcl_slice_A, vcl_slice_A_trans,
                        vcl_B, vcl_B_trans, vcl_slice_C,
                        dtype = dtype)
    except TypeError as e:
        if not "Matrices do not have the same layout" in e.args[0]:
            raise
        else:
            logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=slice, B=range, C=matrix
    print("Now using A=slice, B=range, C=matrix")
    try:
        ret = test_func(epsilon,
                        A, A_trans, B, B_trans, C,
                        vcl_slice_A, vcl_slice_A_trans,
                        vcl_range_B, vcl_range_B_trans, vcl_C,
                        dtype = dtype)
    except TypeError as e:
        if not "Matrices do not have the same layout" in e.args[0]:
            raise
        else:
            logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=slice, B=range, C=range
    print("Now using A=slice, B=range, C=range")
    try:
        ret = test_func(epsilon,
                        A, A_trans, B, B_trans, C,
                        vcl_slice_A, vcl_slice_A_trans,
                        vcl_range_B, vcl_range_B_trans, vcl_range_C,
                        dtype = dtype)
    except TypeError as e:
        if not "Matrices do not have the same layout" in e.args[0]:
            raise
        else:
            logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=slice, B=range, C=slice
    print("Now using A=slice, B=range, C=slice")
    try:
        ret = test_func(epsilon,
                        A, A_trans, B, B_trans, C,
                        vcl_slice_A, vcl_slice_A_trans,
                        vcl_range_B, vcl_range_B_trans, vcl_slice_C,
                        dtype = dtype)
    except TypeError as e:
        if not "Matrices do not have the same layout" in e.args[0]:
            raise
        else:
            logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=slice, B=slice, C=matrix
    print("Now using A=slice, B=slice, C=matrix")
    try:
        ret = test_func(epsilon,
                        A, A_trans, B, B_trans, C,
                        vcl_slice_A, vcl_slice_A_trans,
                        vcl_slice_B, vcl_slice_B_trans, vcl_C,
                        dtype = dtype)
    except TypeError as e:
        if not "Matrices do not have the same layout" in e.args[0]:
            raise
        else:
            logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=slice, B=slice, C=range
    print("Now using A=slice, B=slice, C=range")
    try:
        ret = test_func(epsilon,
                        A, A_trans, B, B_trans, C,
                        vcl_slice_A, vcl_slice_A_trans,
                        vcl_slice_B, vcl_slice_B_trans, vcl_range_C,
                        dtype = dtype)
    except TypeError as e:
        if not "Matrices do not have the same layout" in e.args[0]:
            raise
        else:
            logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=slice, B=slice, C=slice
    print("Now using A=slice, B=slice, C=slice")
    try:
        ret = test_func(epsilon,
                        A, A_trans, B, B_trans, C,
                        vcl_slice_A, vcl_slice_A_trans,
                        vcl_slice_B, vcl_slice_B_trans, vcl_slice_C,
                        dtype = dtype)
    except TypeError as e:
        if not "Matrices do not have the same layout" in e.args[0]:
            raise
        else:
            logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    return os.EX_OK


def test_matrix_solvers(test_func,
                        epsilon, dtype,
                        A_layout = p.ROW_MAJOR, B_layout = p.ROW_MAJOR, 
                        C_layout = p.ROW_MAJOR,
                        size1 = 131, size2 = 131, size3 = 99):

    if size1 != size2:
        raise AttributeError("size1 must equal size2 for solving a system: %d and %d" % (size1, size2))

    if A_layout == p.ROW_MAJOR:
        A_order = 'C'
    else:
        A_order = 'F'

    if B_layout == p.ROW_MAJOR:
        B_order = 'C'
    else:
        B_order = 'F'

    if C_layout == p.ROW_MAJOR:
        C_order = 'C'
    else:
        C_order = 'F'

    # Create reference numpy types
    A = np.empty((size1, size2), dtype = dtype, order = A_order)
    big_A = np.ones((size1 * 4, size2 * 4), dtype = dtype, order = A_order) * 3.1415

    B = np.empty((size2, size3), dtype = dtype, order = B_order)
    big_B = np.ones((size2 * 4, size3 * 4), dtype = dtype, order = B_order) * 42.0

    # Fill A and B with random values
    for i in range(A.shape[0]):
        for j in range(A.shape[1]):
            A[i, j] = random.random()
    for i in range(B.shape[0]):
        for j in range(B.shape[1]):
            B[i, j] = random.random()

    A_trans = A.T
    big_A_trans = big_A.T
    B_trans = B.T
    big_B_trans = big_B.T

    A_upper = A.copy()
    for i in range(A.shape[0]):
        for j in range(A.shape[1]):
            if j < i:
                A_upper[i, j] = 0

    A_lower = A.copy()
    for i in range(A.shape[0]):
        for j in range(A.shape[1]):
            if j > i:
                A_lower[i, j] = 0

    A_unit_upper = A_upper.copy()
    for i in range(A.shape[0]):
        for j in range(A.shape[1]):
            if i == j:
                A_unit_upper[i, j] = 1.0

    A_unit_lower = A_lower.copy()
    for i in range(A.shape[0]):
        for j in range(A.shape[1]):
            if i == j:
                A_unit_lower[i, j] = 1.0

    A_trans_upper = A_trans.copy()
    for i in range(A_trans.shape[0]):
        for j in range(A_trans.shape[1]):
            if j < i:
                A_trans_upper[i, j] = 0

    A_trans_lower = A_trans.copy()
    for i in range(A_trans.shape[0]):
        for j in range(A_trans.shape[1]):
            if j > i:
                A_trans_lower[i, j] = 0

    A_trans_unit_upper = A_trans_upper.copy()
    for i in range(A_trans.shape[0]):
        for j in range(A_trans.shape[1]):
            if i == j:
                A_trans_unit_upper[i, j] = 1.0

    A_trans_unit_lower = A_trans_lower.copy()
    for i in range(A_trans.shape[0]):
        for j in range(A_trans.shape[1]):
            if i == j:
                A_trans_unit_lower[i, j] = 1.0

    # Construct appropriate ViennaCL objects

    #  -- A_upper
    vcl_A_upper = p.Matrix(A_upper, layout = A_layout)

    vcl_big_range_A_upper = p.Matrix(big_A, layout = A_layout)
    vcl_big_range_A_upper[size1:2*size1, size2:2*size2] = vcl_A_upper
    vcl_range_A_upper = vcl_big_range_A_upper[size1:2*size1, size2:2*size2]

    vcl_big_slice_A_upper = p.Matrix(big_A, layout = A_layout)
    vcl_big_slice_A_upper[size1:-size1:2, size2::3] = vcl_A_upper
    vcl_slice_A_upper = vcl_big_slice_A_upper[size1:-size1:2, size2::3]

    vcl_A_trans_upper = p.Matrix(A_trans_upper, layout = A_layout)

    vcl_big_range_A_trans_upper = p.Matrix(big_A_trans, layout = A_layout)
    vcl_big_range_A_trans_upper[size2:2*size2, size1:2*size1] = vcl_A_trans_upper
    vcl_range_A_trans_upper = vcl_big_range_A_trans_upper[size2:2*size2, size1:2*size1]

    vcl_big_slice_A_trans_upper = p.Matrix(big_A_trans, layout = A_layout)
    vcl_big_slice_A_trans_upper[size2:-size2:2, size1::3] = vcl_A_trans_upper
    vcl_slice_A_trans_upper = vcl_big_slice_A_trans_upper[size2:-size2:2, size1::3]

    #  -- A_unit_upper
    vcl_A_unit_upper = p.Matrix(A_unit_upper, layout = A_layout)

    vcl_big_range_A_unit_upper = p.Matrix(big_A, layout = A_layout)
    vcl_big_range_A_unit_upper[size1:2*size1, size2:2*size2] = vcl_A_unit_upper
    vcl_range_A_unit_upper = vcl_big_range_A_unit_upper[size1:2*size1, size2:2*size2]

    vcl_big_slice_A_unit_upper = p.Matrix(big_A, layout = A_layout)
    vcl_big_slice_A_unit_upper[size1:-size1:2, size2::3] = vcl_A_unit_upper
    vcl_slice_A_unit_upper = vcl_big_slice_A_unit_upper[size1:-size1:2, size2::3]

    vcl_A_trans_unit_upper = p.Matrix(A_trans_unit_upper, layout = A_layout)

    vcl_big_range_A_trans_unit_upper = p.Matrix(big_A_trans, layout = A_layout)
    vcl_big_range_A_trans_unit_upper[size2:2*size2, size1:2*size1] = vcl_A_trans_unit_upper
    vcl_range_A_trans_unit_upper = vcl_big_range_A_trans_unit_upper[size2:2*size2, size1:2*size1]

    vcl_big_slice_A_trans_unit_upper = p.Matrix(big_A_trans, layout = A_layout)
    vcl_big_slice_A_trans_unit_upper[size2:-size2:2, size1::3] = vcl_A_trans_unit_upper
    vcl_slice_A_trans_unit_upper = vcl_big_slice_A_trans_unit_upper[size2:-size2:2, size1::3]

    #  -- A_lower
    vcl_A_lower = p.Matrix(A_lower, layout = A_layout)

    vcl_big_range_A_lower = p.Matrix(big_A, layout = A_layout)
    vcl_big_range_A_lower[size1:2*size1, size2:2*size2] = vcl_A_lower
    vcl_range_A_lower = vcl_big_range_A_lower[size1:2*size1, size2:2*size2]

    vcl_big_slice_A_lower = p.Matrix(big_A, layout = A_layout)
    vcl_big_slice_A_lower[size1:-size1:2, size2::3] = vcl_A_lower
    vcl_slice_A_lower = vcl_big_slice_A_lower[size1:-size1:2, size2::3]

    vcl_A_trans_lower = p.Matrix(A_trans_lower, layout = A_layout)

    vcl_big_range_A_trans_lower = p.Matrix(big_A_trans, layout = A_layout)
    vcl_big_range_A_trans_lower[size2:2*size2, size1:2*size1] = vcl_A_trans_lower
    vcl_range_A_trans_lower = vcl_big_range_A_trans_lower[size2:2*size2, size1:2*size1]

    vcl_big_slice_A_trans_lower = p.Matrix(big_A_trans, layout = A_layout)
    vcl_big_slice_A_trans_lower[size2:-size2:2, size1::3] = vcl_A_trans_lower
    vcl_slice_A_trans_lower = vcl_big_slice_A_trans_lower[size2:-size2:2, size1::3]

    #  -- A_unit_lower
    vcl_A_unit_lower = p.Matrix(A_unit_lower, layout = A_layout)

    vcl_big_range_A_unit_lower = p.Matrix(big_A, layout = A_layout)
    vcl_big_range_A_unit_lower[size1:2*size1, size2:2*size2] = vcl_A_unit_lower
    vcl_range_A_unit_lower = vcl_big_range_A_unit_lower[size1:2*size1, size2:2*size2]

    vcl_big_slice_A_unit_lower = p.Matrix(big_A, layout = A_layout)
    vcl_big_slice_A_unit_lower[size1:-size1:2, size2::3] = vcl_A_unit_lower
    vcl_slice_A_unit_lower = vcl_big_slice_A_unit_lower[size1:-size1:2, size2::3]

    vcl_A_trans_unit_lower = p.Matrix(A_trans_unit_lower, layout = A_layout)

    vcl_big_range_A_trans_unit_lower = p.Matrix(big_A_trans, layout = A_layout)
    vcl_big_range_A_trans_unit_lower[size2:2*size2, size1:2*size1] = vcl_A_trans_unit_lower
    vcl_range_A_trans_unit_lower = vcl_big_range_A_trans_unit_lower[size2:2*size2, size1:2*size1]

    vcl_big_slice_A_trans_unit_lower = p.Matrix(big_A_trans, layout = A_layout)
    vcl_big_slice_A_trans_unit_lower[size2:-size2:2, size1::3] = vcl_A_trans_unit_lower
    vcl_slice_A_trans_unit_lower = vcl_big_slice_A_trans_unit_lower[size2:-size2:2, size1::3]

    #  -- B
    vcl_B = p.Matrix(B, layout = B_layout)

    vcl_big_range_B = p.Matrix(big_B, layout = B_layout)
    vcl_big_range_B[size2:2*size2, size3:2*size3] = vcl_B
    vcl_range_B = vcl_big_range_B[size2:2*size2, size3:2*size3]

    vcl_big_slice_B = p.Matrix(big_B, layout = B_layout)
    vcl_big_slice_B[size2:-size2:2, size3::3] = vcl_B
    vcl_slice_B = vcl_big_slice_B[size2:-size2:2, size3::3]

    vcl_B_trans = p.Matrix(B_trans, layout = B_layout)

    vcl_big_range_B_trans = p.Matrix(big_B_trans, layout = B_layout)
    vcl_big_range_B_trans[size3:2*size3, size2:2*size2] = vcl_B_trans
    vcl_range_B_trans = vcl_big_range_B_trans[size3:2*size3, size2:2*size2]

    vcl_big_slice_B_trans = p.Matrix(big_B_trans, layout = B_layout)
    vcl_big_slice_B_trans[size3:-size3:2, size2::3] = vcl_B_trans
    vcl_slice_B_trans = vcl_big_slice_B_trans[size3:-size3:2, size2::3]

    # A=matrix, B=matrix
    print("Now using A=matrix, B=matrix")
    try:
        ret = test_func(epsilon,
                        (A_upper, A_unit_upper, A_lower, A_unit_lower,
                         A_trans_upper, A_trans_unit_upper,
                         A_trans_lower, A_trans_unit_lower),
                        (B, B_trans),
                        (vcl_A_upper, vcl_A_unit_upper,
                         vcl_A_lower, vcl_A_unit_lower,
                         vcl_A_trans_upper, vcl_A_trans_unit_upper,
                         vcl_A_trans_lower, vcl_A_trans_unit_lower),
                        (vcl_B, vcl_B_trans),
                        dtype = dtype)
    except TypeError as e:
        if not "Matrices do not have the same layout" in e.args[0]:
            raise
        else:
            logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=matrix, B=range
    print("Now using A=matrix, B=range")
    try:
        ret = test_func(epsilon,
                        (A_upper, A_unit_upper, A_lower, A_unit_lower,
                         A_trans_upper, A_trans_unit_upper,
                         A_trans_lower, A_trans_unit_lower),
                        (B, B_trans),
                        (vcl_A_upper, vcl_A_unit_upper,
                         vcl_A_lower, vcl_A_unit_lower,
                         vcl_A_trans_upper, vcl_A_trans_unit_upper,
                         vcl_A_trans_lower, vcl_A_trans_unit_lower),
                        (vcl_range_B, vcl_range_B_trans),
                        dtype = dtype)
    except TypeError as e:
        if not "Matrices do not have the same layout" in e.args[0]:
            raise
        else:
            logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=matrix, B=slice
    print("Now using A=matrix, B=slice")
    try:
        ret = test_func(epsilon,
                        (A_upper, A_unit_upper, A_lower, A_unit_lower,
                         A_trans_upper, A_trans_unit_upper,
                         A_trans_lower, A_trans_unit_lower),
                        (B, B_trans),
                        (vcl_A_upper, vcl_A_unit_upper,
                         vcl_A_lower, vcl_A_unit_lower,
                         vcl_A_trans_upper, vcl_A_trans_unit_upper,
                         vcl_A_trans_lower, vcl_A_trans_unit_lower),
                        (vcl_slice_B, vcl_slice_B_trans),
                        dtype = dtype)
    except TypeError as e:
        if not "Matrices do not have the same layout" in e.args[0]:
            raise
        else:
            logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=range, B=matrix
    print("Now using A=range, B=matrix")
    try:
        ret = test_func(epsilon,
                        (A_upper, A_unit_upper, A_lower, A_unit_lower,
                         A_trans_upper, A_trans_unit_upper,
                         A_trans_lower, A_trans_unit_lower),
                        (B, B_trans),
                        (vcl_range_A_upper, vcl_range_A_unit_upper,
                         vcl_range_A_lower, vcl_range_A_unit_lower,
                         vcl_range_A_trans_upper,vcl_range_A_trans_unit_upper,
                         vcl_range_A_trans_lower,vcl_range_A_trans_unit_lower),
                        (vcl_B, vcl_B_trans),
                        dtype = dtype)
    except TypeError as e:
        if not "Matrices do not have the same layout" in e.args[0]:
            raise
        else:
            logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=range, B=range
    print("Now using A=range, B=range")
    try:
        ret = test_func(epsilon,
                        (A_upper, A_unit_upper, A_lower, A_unit_lower,
                         A_trans_upper, A_trans_unit_upper,
                         A_trans_lower, A_trans_unit_lower),
                        (B, B_trans),
                        (vcl_range_A_upper, vcl_range_A_unit_upper,
                         vcl_range_A_lower, vcl_range_A_unit_lower,
                         vcl_range_A_trans_upper,vcl_range_A_trans_unit_upper,
                         vcl_range_A_trans_lower,vcl_range_A_trans_unit_lower),
                        (vcl_range_B, vcl_range_B_trans),
                        dtype = dtype)
    except TypeError as e:
        if not "Matrices do not have the same layout" in e.args[0]:
            raise
        else:
            logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=range, B=slice
    print("Now using A=range, B=slice")
    try:
        ret = test_func(epsilon,
                        (A_upper, A_unit_upper, A_lower, A_unit_lower,
                         A_trans_upper, A_trans_unit_upper,
                         A_trans_lower, A_trans_unit_lower),
                        (B, B_trans),
                        (vcl_range_A_upper, vcl_range_A_unit_upper,
                         vcl_range_A_lower, vcl_range_A_unit_lower,
                         vcl_range_A_trans_upper,vcl_range_A_trans_unit_upper,
                         vcl_range_A_trans_lower,vcl_range_A_trans_unit_lower),
                        (vcl_slice_B, vcl_slice_B_trans),
                        dtype = dtype)
    except TypeError as e:
        if not "Matrices do not have the same layout" in e.args[0]:
            raise
        else:
            logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=slice, B=matrix
    print("Now using A=slice, B=matrix")
    try:
        ret = test_func(epsilon,
                        (A_upper, A_unit_upper, A_lower, A_unit_lower,
                         A_trans_upper, A_trans_unit_upper,
                         A_trans_lower, A_trans_unit_lower),
                        (B, B_trans),
                        (vcl_slice_A_upper, vcl_slice_A_unit_upper,
                         vcl_slice_A_lower, vcl_slice_A_unit_lower,
                         vcl_slice_A_trans_upper,vcl_slice_A_trans_unit_upper,
                         vcl_slice_A_trans_lower,vcl_slice_A_trans_unit_lower),
                        (vcl_B, vcl_B_trans),
                        dtype = dtype)
    except TypeError as e:
        if not "Matrices do not have the same layout" in e.args[0]:
            raise
        else:
            logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=slice, B=range
    print("Now using A=slice, B=range")
    try:
        ret = test_func(epsilon,
                        (A_upper, A_unit_upper, A_lower, A_unit_lower,
                         A_trans_upper, A_trans_unit_upper,
                         A_trans_lower, A_trans_unit_lower),
                        (B, B_trans),
                        (vcl_slice_A_upper, vcl_slice_A_unit_upper,
                         vcl_slice_A_lower, vcl_slice_A_unit_lower,
                         vcl_slice_A_trans_upper,vcl_slice_A_trans_unit_upper,
                         vcl_slice_A_trans_lower,vcl_slice_A_trans_unit_lower),
                        (vcl_range_B, vcl_range_B_trans),
                        dtype = dtype)
    except TypeError as e:
        if not "Matrices do not have the same layout" in e.args[0]:
            raise
        else:
            logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    # A=slice, B=slice
    print("Now using A=slice, B=slice")
    try:
        ret = test_func(epsilon,
                        (A_upper, A_unit_upper, A_lower, A_unit_lower,
                         A_trans_upper, A_trans_unit_upper,
                         A_trans_lower, A_trans_unit_lower),
                        (B, B_trans),
                        (vcl_slice_A_upper, vcl_slice_A_unit_upper,
                         vcl_slice_A_lower, vcl_slice_A_unit_lower,
                         vcl_slice_A_trans_upper,vcl_slice_A_trans_unit_upper,
                         vcl_slice_A_trans_lower,vcl_slice_A_trans_unit_lower),
                        (vcl_slice_B, vcl_slice_B_trans),
                        dtype = dtype)
    except TypeError as e:
        if not "Matrices do not have the same layout" in e.args[0]:
            raise
        else:
            logger.debug("EXCEPTION EXECUTING was: %s" % e.args[0])

    return os.EX_OK


def test_matrix_layout(test_func, test_kernel, epsilon, dtype,
                       size1 = 131, size2 = 131, size3 = 73,
                       num_matrices = 3):
    # A=row, B=row, C=row
    print("///////////////////////////////////////")
    print("/// Now testing A=row, B=row, C=row ///")
    print("///////////////////////////////////////")
    test_func(test_kernel, epsilon, dtype, p.ROW_MAJOR, p.ROW_MAJOR, p.ROW_MAJOR, size1, size2, size3)

    if num_matrices == 3:
        # A=row, B=row, C=col
        print("///////////////////////////////////////")
        print("/// Now testing A=row, B=row, C=col ///")
        print("///////////////////////////////////////")
        test_func(test_kernel, epsilon, dtype, p.ROW_MAJOR, p.ROW_MAJOR, p.COL_MAJOR, size1, size2, size3)

    # A=row, B=col, C=row
    print("///////////////////////////////////////")
    print("/// Now testing A=row, B=col, C=row ///")
    print("///////////////////////////////////////")
    test_func(test_kernel, epsilon, dtype, p.ROW_MAJOR, p.COL_MAJOR, p.ROW_MAJOR, size1, size2, size3)

    if num_matrices == 3:
        # A=row, B=col, C=col
        print("///////////////////////////////////////")
        print("/// Now testing A=row, B=col, C=col ///")
        print("///////////////////////////////////////")
        test_func(test_kernel, epsilon, dtype, p.ROW_MAJOR, p.COL_MAJOR, p.COL_MAJOR, size1, size2, size3)

    # A=col, B=row, C=row
    print("///////////////////////////////////////")
    print("/// Now testing A=col, B=row, C=row ///")
    print("///////////////////////////////////////")
    test_func(test_kernel, epsilon, dtype, p.COL_MAJOR, p.ROW_MAJOR, p.ROW_MAJOR, size1, size2, size3)

    if num_matrices == 3:
        # A=col, B=row, C=col
        print("///////////////////////////////////////")
        print("/// Now testing A=col, B=row, C=col ///")
        print("///////////////////////////////////////")
        test_func(test_kernel, epsilon, dtype, p.COL_MAJOR, p.ROW_MAJOR, p.COL_MAJOR, size1, size2, size3)

    # A=col, B=col, C=row
    print("///////////////////////////////////////")
    print("/// Now testing A=col, B=col, C=row ///")
    print("///////////////////////////////////////")
    test_func(test_kernel, epsilon, dtype, p.COL_MAJOR, p.COL_MAJOR, p.ROW_MAJOR, size1, size2, size3)
    
    if num_matrices == 3:
        # A=col, B=col, C=col
        print("///////////////////////////////////////")
        print("/// Now testing A=col, B=col, C=col ///")
        print("///////////////////////////////////////")
        test_func(test_kernel, epsilon, dtype, p.COL_MAJOR, p.COL_MAJOR, p.COL_MAJOR, size1, size2, size3)

    return os.EX_OK



#!python

"""
Sparse matrix support is still limited in PyViennaCL. The construction
of spare matrices from host data is supported, as is sparse matrix-vector
multiplication and the use of iterative solvers (see iterative-solvers.py
in this directory). However, no other operations have yet been implemented,
and SciPy support is rudimentary.

Here, we demonstrate the construction of a CompressedMatrix instance, and the
calculation of a matrix-vector product.

Other sparse matrix formats are available, such as the CoordinateMatrix,
ELLMatrix and HybridMatrix. They are constructed and used identically to the
CompressedMatrix type.
"""

import pyviennacl as p
import random

# First, we create an empty 5 x 5 CompressedMatrix:
A = p.CompressedMatrix(5, 5)

# Let's set some random values of A.
#
# Importantly, setting individual elements of a PyViennaCL sparse matrix is
# not nearly as expensive as setting individual elements of a dense matrix or
# vector, since in the sparse matrix case, the elements are cached on the host
# and only transferred to the device when they are needed for some computation.
for i in range(6):
    x = random.randrange(0, 4, 1)
    y = random.randrange(0, 4, 1)
    A[x, y] = random.random()

print("A is:\n%s" % A.value)

# Now, let's construct a simple vector of 5 elements.
b = p.Vector(5, 3.142)

print("b is %s" % b)

# Now, represent the product:
c = A * b

# And the result is only computed when we need to print it:
print("A * b = c is %s" % c)




#!python

"""
PyViennaCL provides iterative solvers for sparse and dense linear systems.
The API is documented in ``help(pyviennacl.linalg.solve)``. In particular,
the solver to use is determined by the tag instance supplied to the ``solve``
function.

The iterative solvers have various parameters for tuning the error tolerance,
and various requirements for the form of the system matrix, as described in the
documentation for the corresponding tag classes.

For this reason, we only demonstrate here the use of the GMRES solver for a
general system.
"""

import pyviennacl as p
import numpy as np
import os, random
from util import read_mtx, read_vector

A = read_mtx(os.path.join(os.path.dirname(os.path.realpath(__file__)), "mat65k.mtx"),
             dtype=np.float32)
print("Loaded system matrix")

b = read_vector(os.path.join(os.path.dirname(os.path.realpath(__file__)), "rhs65025.txt"),
                dtype=np.float32)
print("Loaded RHS vector")

# Construct the tag to denote the GMRES solver
tag = p.gmres_tag(tolerance = 1e-5, max_iterations = 150, krylov_dim = 50)

# Solve the system
x = p.solve(A, b, tag)

# Show some info
print("Num. iterations: %s" % tag.iters)
print("Estimated error: %s" % tag.error)




"""
PyViennaCL
==========

This extension provides the Python bindings for the ViennaCL linear
algebra and numerical computation library for GPGPU and heterogeneous
systems. ViennaCL itself is a header-only C++ library, so these bindings
make available to Python programmers ViennaCL's fast OpenCL and CUDA 
algorithms, in a way that is idiomatic and compatible with the Python
community's most popular scientific packages, NumPy and SciPy.

PyViennaCL is divided into five submodules, of which four are designed
for direct use by users:

  * :doc:`pycore`: user-friendly classes for representing the main ViennaCL
    objects, such as Matrix or Vector;
  * :doc:`linalg`: an explicit interface to a number of ViennaCL's linear
    algebra routines, such as matrix solvers and eigenvalue computation;
  * :doc:`vclmath`: convenience functions akin to the standard math module
    (you can also access it as ``pyviennacl.math``);
  * :doc:`util`: utility functions, such as to construct an appropriate
    ViennaCL object from an ndarray (Matrix or Vector), or to provide basic
    debug logging;
  * _viennacl: a raw C++ interface to ViennaCL, with no stable API.

Nonetheless, all of PyViennaCL's functionality is available from the
top-level pyviennacl namespace. So, if you want help on the Matrix class,
you can just run::

  >>> import pyviennacl as p
  >>> help(p.Matrix)                                     # doctest: +SKIP

For help on PyViennaCL's core functionality in general, or
PyViennaCL's high-level linear algebra functions, run::

   >>> help(p.pycore)                                    # doctest: +SKIP

or::

   >>> help(p.linalg)                                    # doctest: +SKIP
"""

from .version import VERSION_TEXT as __version__
from ._viennacl import __version__ as __viennacl_version__

from .pycore import *
from .linalg import *
from .util import *
from .vclmath import *

from . import vclmath as math

# TODO: __all__



"""
This submodule provides convenience functions for performing basic
mathematical operations, akin to the standard Python module `math'.

``pyviennacl.vclmath`` is also accessible under ``pyviennacl.math``,
for simplicity

Convenient equivalences
^^^^^^^^^^^^^^^^^^^^^^^

========= =======================
``abs``   ``pycore.ElementAbs``
``acos``  ``pycore.ElementAcos``
``asin``  ``pycore.ElementAsin``
``atan``  ``pycore.ElementAtan``
``ceil``  ``pycore.ElementCeil``
``cos``   ``pycore.ElementCos``
``cosh``  ``pycore.ElementCosh``
``exp``   ``pycore.ElementExp``
``fabs``  ``pycore.ElementFabs``
``floor`` ``pycore.ElementFloor``
``log``   ``pycore.ElementLog``
``log10`` ``pycore.ElementLog10``
``sin``   ``pycore.ElementSin``
``sinh``  ``pycore.ElementSinh``
``sqrt``  ``pycore.ElementSqrt``
``tan``   ``pycore.ElementTan``
``tanh``  ``pycore.ElementTanh``
========= =======================
"""

from pyviennacl import pycore

abs = pycore.ElementAbs
acos = pycore.ElementAcos
asin = pycore.ElementAsin
atan = pycore.ElementAtan
ceil = pycore.ElementCeil
cos = pycore.ElementCos
cosh = pycore.ElementCosh
exp = pycore.ElementExp
fabs = pycore.ElementFabs
floor = pycore.ElementFloor
log = pycore.ElementLog
log10 = pycore.ElementLog10
sin = pycore.ElementSin
sinh = pycore.ElementSinh
sqrt = pycore.ElementSqrt
tan = pycore.ElementTan
tanh = pycore.ElementTanh




#!/usr/bin/env python

import math
import os
import pyviennacl as p
import sys

from test_common import diff, test_matrix_layout, test_matrix_slice


def run_test(*args, **kwargs):
    """
    A, A_trans, B, B_trans must be numpy array or matrix instances
    """
    epsilon = args[0]
    A = args[1]
    A_trans = args[2]
    B = args[3]
    B_trans = args[4]
    C = args[5]
    vcl_A = args[6]
    vcl_A_trans = args[7]
    vcl_B = args[8]
    vcl_B_trans = args[9]
    vcl_C = args[10]

    act_diff = math.fabs(diff(A, vcl_A))
    if act_diff > epsilon:
        raise Exception("Error copying A")

    act_diff = math.fabs(diff(B, vcl_B))
    if act_diff > epsilon:
        x = p.Matrix(vcl_B.shape, dtype = vcl_B.dtype, layout = p.ROW_MAJOR)
        p.Assign(x, vcl_B).execute()
        print(x.value)
        print(B)
        print(x == B)
        print(act_diff)
        raise Exception("Error copying B")

    #act_diff = math.fabs(diff(C, vcl_C))
    #if act_diff > epsilon:
    #    raise Exception("Error copying C")

    act_diff = math.fabs(diff(A_trans, vcl_A_trans))
    if act_diff > epsilon:
        raise Exception("Error copying A_trans")

    act_diff = math.fabs(diff(B_trans, vcl_B_trans))
    if act_diff > epsilon:
        raise Exception("Error copying B_trans")

    #act_diff = math.fabs(diff(C_trans, vcl_C_trans))
    #if act_diff > epsilon:
    #    raise Exception("Error copying C_trans")

    #A = vcl_A.value
    #A_trans = vcl_A_trans.value
    #B = vcl_B.value
    #B_trans = vcl_B_trans.value
    #C = vcl_C.value
    #C_trans = C.T

    # C +-= A * B
    C = A.dot(B)
    vcl_C = vcl_A * vcl_B
    act_diff = math.fabs(diff(C, vcl_C))
    if (act_diff > epsilon):
        raise Exception("Error at operation: matrix-matrix product; diff = %s"
                        % act_diff)
    print("Test C = A * B passed!")

    C += A.dot(B)
    vcl_C += vcl_A * vcl_B
    act_diff = math.fabs(diff(C, vcl_C))
    if (act_diff > epsilon):
        raise Exception("Error at operation: matrix-matrix product; diff = %s"
                        % act_diff)
    print("Test C += A * B passed!")

    C -= A.dot(B)
    vcl_C -= vcl_A * vcl_B
    act_diff = math.fabs(diff(C, vcl_C))
    if (act_diff > epsilon):
        raise Exception("Error at operation: matrix-matrix product; diff = %s"
                        % act_diff)
    print("Test C -= A * B passed!")

    # C +-= A * trans(B)
    C = A.dot(B_trans.T)
    vcl_C = vcl_A * vcl_B_trans.T
    act_diff = math.fabs(diff(C, vcl_C))
    if (act_diff > epsilon):
        raise Exception("Error at operation: matrix-matrix product; diff = %s"
                        % act_diff)
    print("Test C = A * trans(B) passed!")

    C += A.dot(B_trans.T)
    vcl_C += vcl_A * vcl_B_trans.T
    act_diff = math.fabs(diff(C, vcl_C))
    if (act_diff > epsilon):
        raise Exception("Error at operation: matrix-matrix product; diff = %s"
                        % act_diff)
    print("Test C += A * trans(B) passed!")

    C -= A.dot(B_trans.T)
    vcl_C -= vcl_A * vcl_B_trans.T
    act_diff = math.fabs(diff(C, vcl_C))
    if (act_diff > epsilon):
        raise Exception("Error at operation: matrix-matrix product; diff = %s"
                        % act_diff)
    print("Test C -= A * trans(B) passed!")

    # C +-= trans(A) * B
    C = A_trans.T.dot(B)
    vcl_C = vcl_A_trans.T * vcl_B
    act_diff = math.fabs(diff(C, vcl_C))
    if (act_diff > epsilon):
        raise Exception("Error at operation: matrix-matrix product; diff = %s"
                        % act_diff)
    print("Test C = trans(A) * B passed!")

    C += A_trans.T.dot(B)
    vcl_C += vcl_A_trans.T * vcl_B
    act_diff = math.fabs(diff(C, vcl_C))
    if (act_diff > epsilon):
        raise Exception("Error at operation: matrix-matrix product; diff = %s"
                        % act_diff)
    print("Test C += trans(A) * B passed!")

    C -= A_trans.T.dot(B)
    vcl_C -= vcl_A_trans.T * vcl_B
    act_diff = math.fabs(diff(C, vcl_C))
    if (act_diff > epsilon):
        raise Exception("Error at operation: matrix-matrix product; diff = %s"
                        % act_diff)
    print("Test C -= trans(A) * B passed!")

    # C +-= trans(A) * trans(B)
    C = A_trans.T.dot(B_trans.T)
    vcl_C = vcl_A_trans.T * vcl_B_trans.T
    act_diff = math.fabs(diff(C, vcl_C))
    if (act_diff > epsilon):
        raise Exception("Error at operation: matrix-matrix product; diff = %s"
                        % act_diff)
    print("Test C = trans(A) * trans(B) passed!")

    C += A_trans.T.dot(B_trans.T)
    vcl_C += vcl_A_trans.T * vcl_B_trans.T
    act_diff = math.fabs(diff(C, vcl_C))
    if (act_diff > epsilon):
        raise Exception("Error at operation: matrix-matrix product; diff = %s"
                        % act_diff)
    print("Test C += trans(A) * trans(B) passed!")

    C -= A_trans.T.dot(B_trans.T)
    vcl_C -= vcl_A_trans.T * vcl_B_trans.T
    act_diff = math.fabs(diff(C, vcl_C))
    if (act_diff > epsilon):
        raise Exception("Error at operation: matrix-matrix product; diff = %s"
                        % act_diff)
    print("Test C -= trans(A) * trans(B) passed!")

    return os.EX_OK


def test():
    print("----------------------------------------------")
    print("----------------------------------------------")
    print("## Test :: BLAS 3 routines")
    print("----------------------------------------------")
    print("----------------------------------------------")
    print()
    print("----------------------------------------------")
    print("--- Part 1: Testing matrix-matrix products ---")

    print("*** Using float numeric type ***")
    print("# Testing setup:")
    epsilon = 1.0E-3
    print("  eps:      %s" % epsilon)
    test_matrix_layout(test_matrix_slice, run_test,
                       epsilon, p.float32) #, 11, 11, 11)

    print("*** Using double numeric type ***")
    print("# Testing setup:")
    epsilon = 1.0E-11
    print("  eps:      %s" % epsilon)
    test_matrix_layout(test_matrix_slice, run_test,
                       epsilon, p.float64)

    print("# Test passed")
    
    return os.EX_OK


if __name__ == "__main__":
    sys.exit(test())




#!/usr/bin/env python

import math
import numpy as np
import os
import pyviennacl as p
import random
import sys


def test_scalar(epsilon, dtype):
    """
    Basic arithmetic:
    + add, sub, mul, div
    
    Scalar result types:
    
    Host and Device scalars
    """
    a = dtype(random.random())
    b = dtype(random.random())
    c = dtype(random.random())
    d = dtype(random.random())

    alpha = p.Scalar(a)
    beta = p.Scalar(b)
    gamma = p.HostScalar(c)
    delta = p.HostScalar(d)

    if not alpha == a:
        raise RuntimeError("Failed to initialise device scalar")
    if not beta == b:
        raise RuntimeError("Failed to initialise device scalar")
    if not gamma == c:
        raise RuntimeError("Failed to initialise host scalar")
    if not delta == d:
        raise RuntimeError("Failed to initialise host scalar")
    print("Test: scalar initialisation passed")

    ### Test copy
    A = alpha.copy()
    if A != a:
        raise RuntimeError("Failed to copy device scalar")
    G = gamma.copy()
    if G != c:
        raise RuntimeError("Failed to copy host scalar")
    print("Test: scalar copy passed")
    
    ### Test inter-type initialisation
    A_tmp = A
    A = p.Scalar(G)
    if A != c:
        raise RuntimeError("Failed to initialise device scalar from copied host scalar")
    B = p.HostScalar(beta)
    if B != b:
        raise RuntimeError("Failed to initialise host scalar from device scalar")
    G = p.HostScalar(A_tmp)
    if G != a:
        raise RuntimeError("Failed to initialise host scalar from copied device scalar")
    D = p.Scalar(delta)
    if D != d:
        raise RuntimeError("Failed to initialise device scalar from host scalar")
    print("Test: inter-type scalar initialisation passed")

    ### pyvcl type arithmetic
    X = (a / c) + (b ** a) * (c - d) // b
    Y = (alpha / gamma) + (beta ** alpha) * (gamma - delta) // beta
    X += a
    Y += alpha
    X -= b
    Y -= beta
    X *= c
    Y *= gamma
    X /= d
    Y /= delta
    X **= dtype(2)
    Y **= p.HostScalar(dtype(2))
    X //= Y
    Y //= Y
    if (X - Y) > epsilon:
        raise RuntimeError("Failed basic arithmetic test")
    print("Test: basic arithmetic passed")

    ### Inter-type arithmetic
    X = (a / gamma) + (b ** alpha) * (c - delta) // beta
    Y = (alpha / c) + (beta ** a) * (gamma - d) // b
    X = p.Scalar(X, dtype = dtype)
    X += alpha
    Y += a
    X -= b
    Y -= beta
    X *= gamma
    Y *= c
    X /= d
    Y /= delta
    X **= p.HostScalar(dtype(2))
    Y **= dtype(2)
    if (X - Y) > epsilon:
        raise RuntimeError("Failed inter-type arithmetic test")
    print("Test: inter-type arithmetic passed")

    ### Scalar result type arithmetic
    """
    + Norm_1, Norm_2, Norm_Inf
    + Element* operations?
    + Dot (ie, inner product)
    """
    vec = p.Vector([X, a, beta, c, delta, Y], dtype = dtype)
    r1 = vec.norm(1)
    r2 = vec.norm(2)
    r3 = vec.norm(p.inf)
    r4 = vec.dot(vec * alpha)
    R1 = r1.value
    R2 = r2.value
    R3 = r3.value
    R4 = r4.value
    X = (r1 * a + r2 * beta - r3 / c - r4 // delta) * (r1 + r4 - R3)
    Y = (R1 * a + R2 * beta - R3 / c - R4 // delta) * (R1 + R4 - r3)
    if (X - Y) > epsilon:
        raise RuntimeError("Failed scalar result type arithmetic test")
    print("Test: scalar result type arithmetic passed")


def test():
    print("----------------------------------------------")
    print("----------------------------------------------")
    print("## Test :: Scalar operations")
    print("----------------------------------------------")
    print("----------------------------------------------")
    print()
    print("----------------------------------------------")

    print("*** Using float numeric type ***")
    print("# Testing setup:")
    epsilon = 1.0E-3
    print("  eps:      %s" % epsilon)
    test_scalar(epsilon, p.float32)

    print("*** Using double numeric type ***")
    print("# Testing setup:")
    epsilon = 1.0E-11
    print("  eps:      %s" % epsilon)
    test_scalar(epsilon, p.float64)

    print("# Test passed")
    
    return os.EX_OK


if __name__ == "__main__":
    sys.exit(test())





#!/usr/bin/env python

import math
import numpy as np
import os
import pyviennacl as p
import sys

from test_common import diff, test_matrix_layout, test_matrix_slice

#TODO: Change print statements to log statements

def run_test(*args, **kwargs):
    """
    A, A_trans, B, B_trans must be numpy array or matrix instances
    """
    epsilon = args[0]
    A = args[1]
    A_trans = args[2]
    B = args[3]
    B_trans = args[4]
    C = args[5]
    vcl_A = args[6]
    vcl_A_trans = args[7]
    vcl_B = args[8]
    vcl_B_trans = args[9]
    vcl_C = args[10]

    dtype = kwargs['dtype']

    alpha = p.Scalar(dtype(3.1415))
    beta = p.HostScalar(dtype(2.718))

    # Test initialisers
    # + GPU scalar TODO
    #X = p.Matrix(A.shape, alpha)
    #if not (X == (np.ones(A.shape, dtype = dtype) * alpha.value)).all():
    #    raise RuntimeError("Failed: GPU scalar matrix init")
    #print("Test: initialisation of matrix with GPU scalar passed")

    # + CPU scalar TODO
    Y = p.Matrix(A.shape, beta.value) # TODO
    if not (Y == (np.ones(A.shape, dtype = dtype) * beta.value)).all():
        raise RuntimeError("Failed: CPU scalar matrix init")
    print("Test: initialisation of matrix with CPU scalar passed")

    # + ndarray
    X = p.Matrix(np.ones(A.shape, dtype = dtype) * beta.value)
    if not (X == (np.ones(A.shape, dtype = dtype) * beta.value)).all():
        raise RuntimeError("Failed: ndarray matrix init")
    print("Test: initialisation of matrix with ndarray passed")

    # + Matrix
    X = p.Matrix(Y)
    if not (X == Y).all():
        raise RuntimeError("Failed: Matrix Matrix init")
    print("Test: initialisation of matrix with Matrix passed")

    # + CompressedMatrix -- TODO: sparse matrices + dtypes
    #Y = p.CompressedMatrix(X)
    #X = p.Matrix(Y)
    #if not (X == Y).all():
    #    raise RuntimeError("Failed: Matrix CompressedMatrix init")
    #print("Test: initialisation of matrix with CompressedMatrix passed")
    
    # In-place add
    X = vcl_A.value
    X += vcl_B.value
    vcl_A += vcl_B
    if not (vcl_A == X).all():
        raise RuntimeError("Failed: in-place add")
    print("Test: in-place add passed")

    # Scaled in-place add
    X += alpha.value * vcl_B.value
    vcl_A += alpha * vcl_B
    if not (vcl_A == X).all():
        raise RuntimeError("Failed: scaled in-place add")
    print("Test: scaled in-place add passed")

    # Add
    Y = vcl_A.value + vcl_B.value
    Z = vcl_A + vcl_B
    if not (Y == Z).all():
        raise RuntimeError("Failed: add")
    print("Test: add passed")

    # Scaled add (left)
    Y = dtype(alpha.value) * vcl_B.value + vcl_C.value
    Z = alpha * vcl_B + vcl_C
    act_diff = math.fabs(diff(Y, Z))
    if act_diff > epsilon:
        raise RuntimeError("Failed: scaled add (left)")
    print("Test: scaled add (left) passed")

    # Scaled add (right)
    Y = vcl_B.value + dtype(alpha.value) * vcl_C.value
    Z = vcl_B + alpha * vcl_C
    act_diff = math.fabs(diff(Y, Z))
    if act_diff > epsilon: # (Z == Y).all():
        raise RuntimeError("Failed: scaled add (left)")
    print("Test: scaled add (right) passed")

    # Scaled add (both)
    Y = alpha.value * vcl_B.value + alpha.value * vcl_C.value
    Z = alpha * vcl_B + alpha * vcl_C
    act_diff = math.fabs(diff(Y, Z))
    if act_diff > epsilon:
        raise RuntimeError("Failed: scaled add (both)")
    print("Test: scaled add (both) passed")

    # In-place sub
    X = vcl_A.value
    X -= vcl_B.value
    vcl_A -= vcl_B
    if not (vcl_A == X).all():
        raise RuntimeError("Failed: in-place sub")
    print("Test: in-place sub passed")

    # Scaled in-place sub
    X -= alpha.value * vcl_B.value
    vcl_A -= alpha * vcl_B
    if not (vcl_A == X).all():
        raise RuntimeError("Failed: scaled in-place sub")
    print("Test: scaled in-place sub passed")

    # Sub
    Y = vcl_A.value - vcl_B.value
    Z = vcl_A - vcl_B
    if not (Y == Z).all():
        raise RuntimeError("Failed: sub")
    print("Test: sub passed")

    # Scaled sub (left)
    Y = alpha.value * vcl_B.value - vcl_C.value
    Z = alpha * vcl_B - vcl_C
    act_diff = math.fabs(diff(Y, Z))
    if act_diff > epsilon:
        raise RuntimeError("Failed: scaled sub (left)")
    print("Test: scaled sub (left) passed")

    # Scaled sub (right)
    Y = vcl_B.value - alpha.value * vcl_C.value
    Z = vcl_B - alpha * vcl_C
    act_diff = math.fabs(diff(Y, Z))
    if act_diff > epsilon:
        raise RuntimeError("Failed: scaled sub (right)")
    print("Test: scaled sub (right) passed")

    # Scaled sub (both)
    Y = alpha.value * vcl_B.value - alpha.value * vcl_C.value
    Z = alpha * vcl_B - alpha * vcl_C
    act_diff = math.fabs(diff(Y, Z))
    if act_diff > epsilon:
        raise RuntimeError("Failed: scaled sub (both)")
    print("Test: scaled sub (both) passed")

    # Scalar multiplication (CPU scalar) -- not supported yet
    #gamma_py = beta.value * beta.value
    #gamma_vcl = beta * beta
    # ...
    # Scalar multiplication (GPU scalar)

    # Matrix-vector multiplication
    vec = p.Vector(vcl_A.shape[0], 3.1415, dtype = dtype)
    X = vcl_A * vec
    Y = vcl_A.value.dot(vec.value)
    act_diff = math.fabs(diff(X, Y))
    if act_diff > epsilon:
        raise RuntimeError("Failed: matrix-vector multiplication")
    print("Test: matrix-vector multiplication passed")

    # Matrix divided by scalar
    X = vcl_A.value / alpha.value
    Y = vcl_A / alpha
    act_diff = math.fabs(diff(X, Y))
    if act_diff > epsilon:
        raise RuntimeError("Failed: matrix-scalar division")
    print("Test: matrix-scalar division passed")

    # Binary elementwise operations -- prod and div
    X = vcl_A.value * vcl_B.value
    Y = p.ElementProd(vcl_A, vcl_B)
    act_diff = math.fabs(diff(X, Y))
    if act_diff > epsilon:
        raise RuntimeError("Failed: elementwise matrix-matrix multiplication")
    print("Test: elementwise matrix-matrix multiplication passed")

    X = vcl_A.value ** vcl_B.value
    Y = vcl_A ** vcl_B
    act_diff = math.fabs(diff(X, Y))
    if act_diff > epsilon:
        raise RuntimeError("Failed: elementwise matrix-matrix exponentiation")
    print("Test: elementwise matrix-matrix exponentiation passed")

    X = vcl_A.value / vcl_B.value
    Y = p.ElementDiv(vcl_A, vcl_B)
    act_diff = math.fabs(diff(X, Y))
    if act_diff > epsilon:
        raise RuntimeError("Failed: elementwise matrix-matrix division")
    print("Test: elementwise matrix-matrix division passed")

    # Unary elementwise operations
    # - abs TODO
    #X = abs(vcl_A.value)
    #Y = p.ElementAbs(vcl_A)
    #act_diff = math.fabs(diff(X, Y))
    #if act_diff > epsilon:
    #    raise RuntimeError("Failed: elementwise abs")
    #print("Test: elementwise abs passed")

    # - acos
    X = np.arccos(vcl_A.value)
    Y = p.ElementAcos(vcl_A).result # TODO THIS SHOULDN'T BE REQUIRED
    act_diff = math.fabs(diff(X, Y))
    if act_diff > epsilon:
        raise RuntimeError("Failed: elementwise acos")
    print("Test: elementwise acos passed")

    # - asin
    X = np.arcsin(vcl_A.value)
    Y = p.ElementAsin(vcl_A).result
    act_diff = math.fabs(diff(X, Y))
    if act_diff > epsilon:
        raise RuntimeError("Failed: elementwise asin")
    print("Test: elementwise asin passed")

    # - atan
    X = np.arctan(vcl_A.value)
    Y = p.ElementAtan(vcl_A).result
    act_diff = math.fabs(diff(X, Y))
    if act_diff > epsilon:
        raise RuntimeError("Failed: elementwise atan")
    print("Test: elementwise atan passed")

    # - ceil
    X = np.ceil(vcl_A.value)
    Y = p.ElementCeil(vcl_A).result
    act_diff = math.fabs(diff(X, Y))
    if act_diff > epsilon:
        raise RuntimeError("Failed: elementwise ceil")
    print("Test: elementwise ceil passed")

    # - cos
    X = np.cos(vcl_A.value)
    Y = p.ElementCos(vcl_A).result
    act_diff = math.fabs(diff(X, Y))
    if act_diff > epsilon:
        raise RuntimeError("Failed: elementwise cos")
    print("Test: elementwise cos passed")

    # - cosh
    X = np.cosh(vcl_A.value)
    Y = p.ElementCosh(vcl_A).result
    act_diff = math.fabs(diff(X, Y))
    if act_diff > epsilon:
        raise RuntimeError("Failed: elementwise cosh")
    print("Test: elementwise cosh passed")

    # - exp
    X = np.exp(vcl_A.value)
    Y = p.ElementExp(vcl_A).result
    act_diff = math.fabs(diff(X, Y))
    if act_diff > epsilon:
        raise RuntimeError("Failed: elementwise exp")
    print("Test: elementwise exp passed")

    # - fabs
    X = np.fabs(vcl_A.value)
    Y = p.ElementFabs(vcl_A).result
    act_diff = math.fabs(diff(X, Y))
    if act_diff > epsilon:
        raise RuntimeError("Failed: elementwise fabs")
    print("Test: elementwise fabs passed")

    # - floor
    X = np.floor(vcl_A.value)
    Y = p.ElementFloor(vcl_A).result
    act_diff = math.fabs(diff(X, Y))
    if act_diff > epsilon:
        raise RuntimeError("Failed: elementwise floor")
    print("Test: elementwise floor passed")

    # - log
    X = np.log(vcl_A.value)
    Y = p.ElementLog(vcl_A).result
    act_diff = math.fabs(diff(X, Y))
    if act_diff > epsilon:
        raise RuntimeError("Failed: elementwise log")
    print("Test: elementwise log passed")

    # - log10
    X = np.log10(vcl_A.value)
    Y = p.ElementLog10(vcl_A).result
    act_diff = math.fabs(diff(X, Y))
    if act_diff > epsilon:
        raise RuntimeError("Failed: elementwise log10")
    print("Test: elementwise log10 passed")

    # - sin
    X = np.sin(vcl_A.value)
    Y = p.ElementSin(vcl_A).result
    act_diff = math.fabs(diff(X, Y))
    if act_diff > epsilon:
        raise RuntimeError("Failed: elementwise sin")
    print("Test: elementwise sin passed")

    # - sinh
    X = np.sinh(vcl_A.value)
    Y = p.ElementSinh(vcl_A).result
    act_diff = math.fabs(diff(X, Y))
    if act_diff > epsilon:
        raise RuntimeError("Failed: elementwise sinh")
    print("Test: elementwise sinh passed")

    # - sqrt
    X = np.sqrt(vcl_A.value)
    Y = p.ElementSqrt(vcl_A).result
    act_diff = math.fabs(diff(X, Y))
    if act_diff > epsilon:
        raise RuntimeError("Failed: elementwise sqrt")
    print("Test: elementwise sqrt passed")

    # - tan
    X = np.tan(vcl_A.value)
    Y = p.ElementTan(vcl_A).result
    act_diff = math.fabs(diff(X, Y))
    if act_diff > epsilon:
        raise RuntimeError("Failed: elementwise tan")
    print("Test: elementwise tan passed")

    # - tanh
    X = np.tanh(vcl_A.value)
    Y = p.ElementTanh(vcl_A).result
    act_diff = math.fabs(diff(X, Y))
    if act_diff > epsilon:
        raise RuntimeError("Failed: elementwise tanh")
    print("Test: elementwise tanh passed")

    # - trans TODO ?!!!
    #X = vcl_A.value.T
    #Y = vcl_A.T.result #p.Trans(vcl_A).result
    #act_diff = math.fabs(diff(X, Y))
    #if act_diff > epsilon:
    #    raise RuntimeError("Failed: elementwise trans")
    #print("Test: elementwise trans passed")

    # - norm1 -- TODO ONLY FOR VECTORS
    # - norm2 -- TODO ONLY FOR VECTORS
    # - norm_inf -- TODO ONLY FOR VECTORS

    return os.EX_OK


def test():
    print("----------------------------------------------")
    print("----------------------------------------------")
    print("## Test :: BLAS 3 routines")
    print("----------------------------------------------")
    print("----------------------------------------------")
    print()
    print("----------------------------------------------")
    print("--- Part 1: Testing matrix-matrix products ---")

    print("*** Using float numeric type ***")
    print("# Testing setup:")
    epsilon = 1.0E-3
    print("  eps:      %s" % epsilon)
    test_matrix_layout(test_matrix_slice, run_test,
                       epsilon, p.float32, 11, 11, 11)

    print("*** Using double numeric type ***")
    print("# Testing setup:")
    epsilon = 1.0E-11
    print("  eps:      %s" % epsilon)
    test_matrix_layout(test_matrix_slice, run_test,
                       epsilon, p.float64, 11, 11, 11)

    print("# Test passed")
    
    return os.EX_OK


if __name__ == "__main__":
    sys.exit(test())




import setuptools  # noqa
from setuptools import Extension


def count_down_delay(delay):
    from time import sleep
    import sys
    while delay:
        sys.stdout.write("Continuing in %d seconds...   \r" % delay)
        sys.stdout.flush()
        delay -= 1
        sleep(1)
    print("")

DASH_SEPARATOR = 75 * "-"


def setup(*args, **kwargs):
    from setuptools import setup
    try:
        setup(*args, **kwargs)
    except KeyboardInterrupt:
        raise
    except SystemExit:
        raise
    except:
        print(DASH_SEPARATOR)
        print("Sorry, your build failed. Try rerunning configure.py with "
                "different options.")
        print(DASH_SEPARATOR)
        raise


class NumpyExtension(Extension):
    # nicked from
    # http://mail.python.org/pipermail/distutils-sig/2007-September/008253.html
    # solution by Michael Hoffmann
    def __init__(self, *args, **kwargs):
        Extension.__init__(self, *args, **kwargs)
        self._include_dirs = self.include_dirs
        del self.include_dirs  # restore overwritten property

    def get_numpy_incpath(self):
        from imp import find_module
        # avoid actually importing numpy, it screws up distutils
        file, pathname, descr = find_module("numpy")
        from os.path import join
        return join(pathname, "core", "include")

    def get_include_dirs(self):
        return self._include_dirs + [self.get_numpy_incpath()]

    def set_include_dirs(self, value):
        self._include_dirs = value

    def del_include_dirs(self):
        pass

    include_dirs = property(get_include_dirs, set_include_dirs, del_include_dirs)


class PyUblasExtension(NumpyExtension):
    def get_module_include_path(self, name):
        from pkg_resources import Requirement, resource_filename
        return resource_filename(Requirement.parse(name), "%s/include" % name)

    @property
    def include_dirs(self):
        return self._include_dirs + [
                self.get_numpy_incpath(),
                self.get_module_include_path("pyublas"),
                ]


class HedgeExtension(PyUblasExtension):
    @property
    def include_dirs(self):
        return self._include_dirs + [
                self.get_numpy_incpath(),
                self.get_module_include_path("pyublas"),
                self.get_module_include_path("hedge"),
                ]


# {{{ tools

def flatten(list):
    """For an iterable of sub-iterables, generate each member of each
    sub-iterable in turn, i.e. a flattened version of that super-iterable.

    Example: Turn [[a,b,c],[d,e,f]] into [a,b,c,d,e,f].
    """
    for sublist in list:
        for j in sublist:
            yield j


def humanize(sym_str):
    words = sym_str.lower().replace("_", " ").split(" ")
    return " ".join([word.capitalize() for word in words])

# }}}


# {{{ siteconf handling

def get_config(schema=None, warn_about_no_config=True):
    if schema is None:
        from setup import get_config_schema
        schema = get_config_schema()

    if (not schema.have_config() and not schema.have_global_config()
            and warn_about_no_config):
        print("*************************************************************")
        print("*** I have detected that you have not run configure.py.")
        print("*************************************************************")
        print("*** Additionally, no global config files were found.")
        print("*** I will go ahead with the default configuration.")
        print("*** In all likelihood, this will not work out.")
        print("*** ")
        print("*** See README_SETUP.txt for more information.")
        print("*** ")
        print("*** If the build does fail, just re-run configure.py with the")
        print("*** correct arguments, and then retry. Good luck!")
        print("*************************************************************")
        print("*** HIT Ctrl-C NOW IF THIS IS NOT WHAT YOU WANT")
        print("*************************************************************")

        count_down_delay(delay=10)

    return expand_options(schema.read_config())


def hack_distutils(debug=False, fast_link=True, what_opt=3):
    # hack distutils.sysconfig to eliminate debug flags
    # stolen from mpi4py

    def remove_prefixes(optlist, bad_prefixes):
        for bad_prefix in bad_prefixes:
            for i, flag in enumerate(optlist):
                if flag.startswith(bad_prefix):
                    optlist.pop(i)
                    break
        return optlist

    import sys
    if not sys.platform.lower().startswith("win"):
        from distutils import sysconfig

        cvars = sysconfig.get_config_vars()
        cflags = cvars.get('OPT')
        if cflags:
            cflags = remove_prefixes(cflags.split(),
                    ['-g', '-O', '-Wstrict-prototypes', '-DNDEBUG'])
            if debug:
                cflags.append("-g")
            else:
                if what_opt is None:
                    pass
                else:
                    cflags.append("-O%s" % what_opt)
                    cflags.append("-DNDEBUG")

            cvars['OPT'] = str.join(' ', cflags)
            cvars["CFLAGS"] = cvars["BASECFLAGS"] + " " + cvars["OPT"]

        if fast_link:
            for varname in ["LDSHARED", "BLDSHARED"]:
                ldsharedflags = cvars.get(varname)
                if ldsharedflags:
                    ldsharedflags = remove_prefixes(ldsharedflags.split(),
                            ['-Wl,-O'])
                    cvars[varname] = str.join(' ', ldsharedflags)

# }}}


# {{{ configure guts

def default_or(a, b):
    if a is None:
        return b
    else:
        return a


def expand_str(s, options):
    import re

    def my_repl(match):
        sym = match.group(1)
        try:
            repl = options[sym]
        except KeyError:
            from os import environ
            repl = environ[sym]

        return expand_str(repl, options)

    return re.subn(r"\$\{([a-zA-Z0-9_]+)\}", my_repl, s)[0]


def expand_value(v, options):
    if isinstance(v, str):
        return expand_str(v, options)
    elif isinstance(v, list):
        result = []
        for i in v:
            try:
                exp_i = expand_value(i, options)
            except:
                pass
            else:
                result.append(exp_i)

        return result
    else:
        return v


def expand_options(options):
    return dict(
            (k, expand_value(v, options)) for k, v in options.items())


class ConfigSchema:
    def __init__(self, options, conf_file="siteconf.py", conf_dir="."):
        self.optdict = dict((opt.name, opt) for opt in options)
        self.options = options
        self.conf_dir = conf_dir
        self.conf_file = conf_file

        from os.path import expanduser
        self.user_conf_file = expanduser("~/.aksetup-defaults.py")

        import sys
        if not sys.platform.lower().startswith("win"):
            self.global_conf_file = "/etc/aksetup-defaults.py"
        else:
            self.global_conf_file = None

    def get_conf_file(self):
        import os
        return os.path.join(self.conf_dir, self.conf_file)

    def set_conf_dir(self, conf_dir):
        self.conf_dir = conf_dir

    def get_default_config(self):
        return dict((opt.name, opt.default) for opt in self.options)

    def read_config_from_pyfile(self, filename):
        result = {}
        filevars = {}
        infile = open(filename, "r")
        try:
            contents = infile.read()
        finally:
            infile.close()

        exec(compile(contents, filename, "exec"), filevars)

        for key, value in filevars.items():
            if key in self.optdict:
                result[key] = value

        return result

    def update_conf_file(self, filename, config):
        result = {}
        filevars = {}

        try:
            exec(compile(open(filename, "r").read(), filename, "exec"), filevars)
        except IOError:
            pass

        if "__builtins__" in filevars:
            del filevars["__builtins__"]

        for key, value in config.items():
            if value is not None:
                filevars[key] = value

        keys = filevars.keys()
        keys.sort()

        outf = open(filename, "w")
        for key in keys:
            outf.write("%s = %s\n" % (key, repr(filevars[key])))
        outf.close()

        return result

    def update_user_config(self, config):
        self.update_conf_file(self.user_conf_file, config)

    def update_global_config(self, config):
        if self.global_conf_file is not None:
            self.update_conf_file(self.global_conf_file, config)

    def get_default_config_with_files(self):
        result = self.get_default_config()

        import os

        confignames = []
        if self.global_conf_file is not None:
            confignames.append(self.global_conf_file)
        confignames.append(self.user_conf_file)

        for fn in confignames:
            if os.access(fn, os.R_OK):
                result.update(self.read_config_from_pyfile(fn))

        return result

    def have_global_config(self):
        import os
        result = os.access(self.user_conf_file, os.R_OK)

        if self.global_conf_file is not None:
            result = result or os.access(self.global_conf_file, os.R_OK)

        return result

    def have_config(self):
        import os
        return os.access(self.get_conf_file(), os.R_OK)

    def read_config(self, warn_if_none=True):
        import os
        cfile = self.get_conf_file()

        result = self.get_default_config_with_files()
        if os.access(cfile, os.R_OK):
            filevars = {}
            exec(compile(open(cfile, "r").read(), cfile, "exec"), filevars)

            for key, value in filevars.items():
                if key in self.optdict:
                    result[key] = value
                elif key == "__builtins__":
                    pass
                else:
                    raise KeyError("invalid config key in %s: %s" % (
                            cfile, key))

        return result

    def add_to_configparser(self, parser, def_config=None):
        if def_config is None:
            def_config = self.get_default_config_with_files()

        for opt in self.options:
            default = default_or(def_config.get(opt.name), opt.default)
            opt.add_to_configparser(parser, default)

    def get_from_configparser(self, options):
        result = {}
        for opt in self.options:
            result[opt.name] = opt.take_from_configparser(options)
        return result

    def write_config(self, config):
        outf = open(self.get_conf_file(), "w")
        for opt in self.options:
            value = config[opt.name]
            if value is not None:
                outf.write("%s = %s\n" % (opt.name, repr(config[opt.name])))
        outf.close()

    def make_substitutions(self, config):
        return dict((opt.name, opt.value_to_str(config[opt.name]))
                for opt in self.options)


class Option(object):
    def __init__(self, name, default=None, help=None):
        self.name = name
        self.default = default
        self.help = help

    def as_option(self):
        return self.name.lower().replace("_", "-")

    def metavar(self):
        last_underscore = self.name.rfind("_")
        return self.name[last_underscore+1:]

    def get_help(self, default):
        result = self.help
        if self.default:
            result += " (default: %s)" % self.value_to_str(
                    default_or(default, self.default))
        return result

    def value_to_str(self, default):
        return default

    def add_to_configparser(self, parser, default=None):
        default = default_or(default, self.default)
        default_str = self.value_to_str(default)
        parser.add_option(
            "--" + self.as_option(), dest=self.name,
            default=default_str,
            metavar=self.metavar(), help=self.get_help(default))

    def take_from_configparser(self, options):
        return getattr(options, self.name)


class Switch(Option):
    def add_to_configparser(self, parser, default=None):
        if not isinstance(self.default, bool):
            raise ValueError("Switch options must have a default")

        if default is None:
            default = self.default

        option_name = self.as_option()

        if default:
            option_name = "no-" + option_name
            action = "store_false"
        else:
            action = "store_true"

        parser.add_option(
            "--" + option_name,
            dest=self.name,
            help=self.get_help(default),
            default=default,
            action=action)


class StringListOption(Option):
    def value_to_str(self, default):
        if default is None:
            return None

        return ",".join([str(el).replace(",", r"\,") for el in default])

    def get_help(self, default):
        return Option.get_help(self, default) + " (several ok)"

    def take_from_configparser(self, options):
        opt = getattr(options, self.name)
        if opt is None:
            return None
        else:
            if opt:
                import re
                sep = re.compile(r"(?<!\\),")
                result = sep.split(opt)
                result = [i.replace(r"\,", ",") for i in result]
                return result
            else:
                return []


class IncludeDir(StringListOption):
    def __init__(self, lib_name, default=None, human_name=None, help=None):
        StringListOption.__init__(self, "%s_INC_DIR" % lib_name, default,
                help=help or ("Include directories for %s"
                % (human_name or humanize(lib_name))))


class LibraryDir(StringListOption):
    def __init__(self, lib_name, default=None, human_name=None, help=None):
        StringListOption.__init__(self, "%s_LIB_DIR" % lib_name, default,
                help=help or ("Library directories for %s"
                % (human_name or humanize(lib_name))))


class Libraries(StringListOption):
    def __init__(self, lib_name, default=None, human_name=None, help=None):
        StringListOption.__init__(self, "%s_LIBNAME" % lib_name, default,
                help=help or ("Library names for %s (without lib or .so)"
                % (human_name or humanize(lib_name))))


class BoostLibraries(Libraries):
    def __init__(self, lib_base_name, default_lib_name=None):
        if default_lib_name is None:
            if lib_base_name == "python":
                import sys
                default_lib_name = "boost_python-py%d%d" % sys.version_info[:2]
            else:
                default_lib_name = "boost_%s" % lib_base_name

        Libraries.__init__(self, "BOOST_%s" % lib_base_name.upper(),
                [default_lib_name],
                help="Library names for Boost C++ %s library (without lib or .so)"
                    % humanize(lib_base_name))


def set_up_shipped_boost_if_requested(project_name, conf, source_path=None,
        boost_chrono=False):
    """Set up the package to use a shipped version of Boost.

    Return a tuple of a list of extra C files to build and extra
    defines to be used.

    :arg boost_chrono: one of *False* and ``"header_only"``
        (only relevant in shipped mode)
    """
    from os.path import exists
    import sys

    if source_path is None:
        source_path = "bpl-subset/bpl_subset"

    if conf["USE_SHIPPED_BOOST"]:
        if not exists("%s/boost/version.hpp" % source_path):
            print(DASH_SEPARATOR)
            print("The shipped Boost library was not found, but "
                    "USE_SHIPPED_BOOST is True.")
            print("(The files should be under %s/.)" % source_path)
            print(DASH_SEPARATOR)
            print("If you got this package from git, you probably want to do")
            print("")
            print(" $ git submodule update --init")
            print("")
            print("to fetch what you are presently missing. If you got this from")
            print("a distributed package on the net, that package is broken and")
            print("should be fixed. For now, I will turn off 'USE_SHIPPED_BOOST'")
            print("to try and see if the build succeeds that way, but in the long")
            print("run you might want to either get the missing bits or turn")
            print("'USE_SHIPPED_BOOST' off.")
            print(DASH_SEPARATOR)
            conf["USE_SHIPPED_BOOST"] = False

            count_down_delay(delay=10)

    if conf["USE_SHIPPED_BOOST"]:
        conf["BOOST_INC_DIR"] = [source_path]
        conf["BOOST_LIB_DIR"] = []
        conf["BOOST_PYTHON_LIBNAME"] = []
        conf["BOOST_THREAD_LIBNAME"] = []

        from glob import glob
        source_files = (glob(source_path + "/libs/*/*/*/*.cpp")
                + glob(source_path + "/libs/*/*/*.cpp")
                + glob(source_path + "/libs/*/*.cpp"))

        # make sure next line succeeds even on Windows
        source_files = [f.replace("\\", "/") for f in source_files]

        source_files = [f for f in source_files
                if not f.startswith(source_path + "/libs/thread/src")]

        if sys.platform == "win32":
            source_files += glob(
                    source_path + "/libs/thread/src/win32/*.cpp")
            source_files += glob(
                    source_path + "/libs/thread/src/tss_null.cpp")
        else:
            source_files += glob(
                    source_path + "/libs/thread/src/pthread/*.cpp")

        source_files = [f for f in source_files
                if not f.endswith("once_atomic.cpp")]

        from os.path import isdir
        main_boost_inc = source_path + "/boost"
        bpl_project_boost_inc = source_path + "/%sboost" % project_name

        if not isdir(bpl_project_boost_inc):
            try:
                from os import symlink
                symlink("boost", bpl_project_boost_inc)
            except (ImportError, OSError):
                from shutil import copytree
                print("Copying files, hang on... (do not interrupt)")
                copytree(main_boost_inc, bpl_project_boost_inc)

        defines = {
                # do not pick up libboost link dependency on windows
                "BOOST_ALL_NO_LIB": 1,
                "BOOST_THREAD_BUILD_DLL": 1,

                "BOOST_MULTI_INDEX_DISABLE_SERIALIZATION": 1,
                "BOOST_PYTHON_SOURCE": 1,
                "boost": '%sboost' % project_name,
                }

        if boost_chrono is False:
            defines["BOOST_THREAD_DONT_USE_CHRONO"] = 1
        elif boost_chrono == "header_only":
            defines["BOOST_CHRONO_HEADER_ONLY"] = 1
        else:
            raise ValueError("invalid value of 'boost_chrono'")

        return (source_files, defines)
    else:
        return [], {}


def make_boost_base_options():
    return [
        IncludeDir("BOOST", []),
        LibraryDir("BOOST", []),
        Option("BOOST_COMPILER", default="gcc43",
            help="The compiler with which Boost C++ was compiled, e.g. gcc43"),
        ]


def configure_frontend():
    from optparse import OptionParser

    from setup import get_config_schema
    schema = get_config_schema()
    if schema.have_config():
        print("************************************************************")
        print("*** I have detected that you have already run configure.")
        print("*** I'm taking the configured values as defaults for this")
        print("*** configure run. If you don't want this, delete the file")
        print("*** %s." % schema.get_conf_file())
        print("************************************************************")

    import sys

    description = "generate a configuration file for this software package"
    parser = OptionParser(description=description)
    parser.add_option(
            "--python-exe", dest="python_exe", default=sys.executable,
            help="Which Python interpreter to use", metavar="PATH")

    parser.add_option("--prefix", default=None,
            help="Ignored")
    parser.add_option("--enable-shared", help="Ignored", action="store_false")
    parser.add_option("--disable-static", help="Ignored", action="store_false")
    parser.add_option("--update-user",
            help="Update user config file (%s)" % schema.user_conf_file,
            action="store_true")
    parser.add_option("--update-global",
            help="Update global config file (%s)" % schema.global_conf_file,
            action="store_true")

    schema.add_to_configparser(parser, schema.read_config())

    options, args = parser.parse_args()

    config = schema.get_from_configparser(options)
    schema.write_config(config)

    if options.update_user:
        schema.update_user_config(config)

    if options.update_global:
        schema.update_global_config(config)

    import os
    if os.access("Makefile.in", os.F_OK):
        substs = schema.make_substitutions(config)
        substs["PYTHON_EXE"] = options.python_exe

        substitute(substs, "Makefile")


def substitute(substitutions, fname):
    import re
    var_re = re.compile(r"\$\{([A-Za-z_0-9]+)\}")
    string_var_re = re.compile(r"\$str\{([A-Za-z_0-9]+)\}")

    fname_in = fname+".in"
    lines = open(fname_in, "r").readlines()
    new_lines = []
    for l in lines:
        made_change = True
        while made_change:
            made_change = False
            match = var_re.search(l)
            if match:
                varname = match.group(1)
                l = l[:match.start()] + str(substitutions[varname]) + l[match.end():]
                made_change = True

            match = string_var_re.search(l)
            if match:
                varname = match.group(1)
                subst = substitutions[varname]
                if subst is None:
                    subst = ""
                else:
                    subst = '"%s"' % subst

                l = l[:match.start()] + subst + l[match.end():]
                made_change = True
        new_lines.append(l)
    new_lines.insert(1, "# DO NOT EDIT THIS FILE -- "
            "it was generated by configure.py\n")
    import sys
    new_lines.insert(2, "# %s\n" % (" ".join(sys.argv)))
    open(fname, "w").write("".join(new_lines))

    from os import stat, chmod
    infile_stat_res = stat(fname_in)
    chmod(fname, infile_stat_res.st_mode)


def _run_git_command(cmd):
    git_error = None
    from subprocess import Popen, PIPE
    try:
        popen = Popen(["git"] + cmd, stdout=PIPE)
        stdout, stderr = popen.communicate()
        if popen.returncode != 0:
            git_error = "git returned error code %d: %s" % (popen.returncode, stderr)
    except OSError:
        git_error = "(OS error, likely git not found)"

    if git_error is not None:
        print(DASH_SEPARATOR)
        print("Trouble invoking git")
        print(DASH_SEPARATOR)
        print("The package directory appears to be a git repository, but I could")
        print("not invoke git to check whether my submodules are up to date.")
        print("")
        print("The error was:")
        print(git_error)
        print("Hit Ctrl-C now if you'd like to think about the situation.")
        print(DASH_SEPARATOR)
        count_down_delay(delay=5)
    return stdout.decode("ascii"), git_error


def check_git_submodules():
    from os.path import isdir
    if not isdir(".git"):
        # not a git repository
        return
    if isdir("../.repo"):
        # assume repo is in charge and bail
        return

    stdout, git_error = _run_git_command(["submodule", "status"])
    if git_error is not None:
        return

    pkg_warnings = []

    lines = stdout.split("\n")
    for l in lines:
        if not l.strip():
            continue

        status = l[0]
        sha, package = l[1:].split(" ", 1)

        if package == "bpl-subset" or (
                package.startswith("boost") and package.endswith("subset")):
            # treated separately
            continue

        if status == "+":
            pkg_warnings.append("version of '%s' is not what this "
                    "outer package wants" % package)
        elif status == "-":
            pkg_warnings.append("subpackage '%s' is not initialized"
                    % package)
        elif status == " ":
            pass
        else:
            pkg_warnings.append("subpackage '%s' has unrecognized status '%s'"
                    % package)

    if pkg_warnings:
            print(DASH_SEPARATOR)
            print("git submodules are not up-to-date or in odd state")
            print(DASH_SEPARATOR)
            print("If this makes no sense, you probably want to say")
            print("")
            print(" $ git submodule update --init")
            print("")
            print("to fetch what you are presently missing and "
                    "move on with your life.")
            print("If you got this from a distributed package on the "
                    "net, that package is")
            print("broken and should be fixed. Please inform whoever "
                    "gave you this package.")
            print("")
            print("These issues were found:")
            for w in pkg_warnings:
                print("  %s" % w)
            print("")
            print("I will try to initialize the submodules for you "
                    "after a short wait.")
            print(DASH_SEPARATOR)
            print("Hit Ctrl-C now if you'd like to think about the situation.")
            print(DASH_SEPARATOR)

            from os.path import exists
            if not exists(".dirty-git-ok"):
                count_down_delay(delay=10)
                stdout, git_error = _run_git_command(
                        ["submodule", "update", "--init"])
                if git_error is None:
                    print(DASH_SEPARATOR)
                    print("git submodules initialized successfully")
                    print(DASH_SEPARATOR)



#!python

"""
PyViennaCL allows you to access and manipulate submatrices and subvectors using
the usual Pythonic apparatus of slices and ranges of objects. ViennaCL provides
object proxies to allow us to do these sub-manipulations in place.

Here, we give some basic examples.
"""

import pyviennacl as p
import numpy as np

# Create some small, simple Vector and Matrix instances
x = p.Vector(6, 1.0)
a = p.Matrix(6, 6, 1.0)

print("x is %s" % x)
print("a is\n%s" % a)

# Scale the first half of the Vector x
x[0:3] *= 2.0

# Show the new x
print("x is now %s" % x)

# Create a smaller matrix from a submatrix of a
b = a[3:6, 3:6] * 4.0

# Set the upper-left corner of the matrix to 4.0s
a[0:3, 0:3] = b

# Show the new a
print("a is now\n%s" % a)

# Represent an operation on a
b = p.sqrt(a)

# Manipulate submatrices of b
b[0:3, 3:6] += b[0:3, 0:3]
b[3:6, 3:6] += b[0:3, 3:6]

# Show b
print("b is\n%s" % b)

# We can also manipulate slices of matrices and of submatrices
c = b[0:6, 2:6]
c[0:6:2, 0:4:2] = c[0:6:2, 0:4:2] * 10.0

# Show b after the proxy update via c
print("b is now\n%s" % b)

# We can do similarly for vectors
x[0:6:2] = x[3:6] * 10.0

# Show x
print("x is now %s" % x)

